@@ -1,6 +1,6 @@
 /* ex: set ts=4: */
 /***************************************************************************
- *  gst-misc.c
+ *  gst-misc-0.10.c
  *
  *  Copyright (C) 2005 Novell
  *  Written by Aaron Bockover (aaron@aaronbock.net)
@@ -37,17 +37,41 @@
 #include <string.h>
 
 #include <gst/gst.h>
-#include <gst/gconf/gconf.h>
+
+#include "gst-mbtrm.h"
 
 static gboolean gstreamer_initialized = FALSE;
 
+static gboolean
+gst_mbtrm_register_elements(GstPlugin *plugin)
+{
+    return gst_element_register(plugin, "mbtrm",
+        GST_RANK_NONE, GST_TYPE_MBTRM);
+}
+
+static GstPluginDesc gst_mbtrm_plugin_desc = {
+    GST_VERSION_MAJOR,
+    GST_VERSION_MINOR,
+    "mbtrm",
+    "Private MusicBrainz TRM element",
+    gst_mbtrm_register_elements,
+    "0.10.10",
+    "LGPL",
+    "libbanshee",
+    "Banshee",
+    "http://banshee-project.org/",
+    GST_PADDING_INIT
+};
+
 void gstreamer_initialize()
 {
-    if(gstreamer_initialized)
+    if(gstreamer_initialized) {
         return;
+    }
 
     gst_init(NULL, NULL);
-
+    _gst_plugin_register_static(&gst_mbtrm_plugin_desc);
+    
     gstreamer_initialized = TRUE;
 }
 
@@ -62,81 +86,60 @@ gstreamer_test_encoder(gchar *encoder_pipeline)
     element = gst_parse_launch(pipeline, &error);
     g_free(pipeline);
     
-    if(element != NULL)
+    if(element != NULL) {
         gst_object_unref(GST_OBJECT(element));
-    
-    return error == NULL;
-    
-    /*GstElement *element = NULL;
-    gchar *pipeline;
-    
-    pipeline = g_strdup_printf("audioconvert ! %s", encoder_pipeline);
-    element = gst_gconf_render_bin_from_description(pipeline);
-    g_free(pipeline);
-    
-    if(element) {
-        gst_object_unref(GST_OBJECT(element));
-        return TRUE;
     }
     
-    return FALSE;*/
+    return error == NULL;
 }
 
 static void
-gst_type_found_cb(GstElement *typefind, guint probability, GstCaps *caps, 
-    gchar **type)
+gst_typefind_type_found_callback(GstElement *typefind, guint probability, 
+    GstCaps *caps, gchar **type)
 {
     *type = gst_caps_to_string(caps);    
 }
 
-static void
-gst_error_cb(GstElement *pipeline, GstElement *source, GError *error,
-    gchar *debug, gchar **data)
-{
-    *data = "error";
-}
-
 gchar *
 gstreamer_detect_mimetype(const gchar *uri)
 {
-    GstElement *pipeline;
+    return NULL;
+    /*GstElement *pipeline;
     GstElement *source;
     GstElement *typefind;
+    GstElement *fakesink;
     gchar *mimetype = NULL;
 
     pipeline = gst_pipeline_new("new");
-    g_signal_connect(pipeline, "error", G_CALLBACK(gst_error_cb), &mimetype);
-
+        
     source = gst_element_factory_make("gnomevfssrc", "source");
     typefind = gst_element_factory_make("typefind", "typefind");
+    fakesink = gst_element_factory_make("fakesink", "fakesink");
 
     if(source == NULL || typefind == NULL) {
-        gst_object_unref(GST_OBJECT(pipeline));
+        gst_object_unref(pipeline);
         return NULL;
     }
 
-    g_object_set(G_OBJECT(source), "location", uri, NULL);
-    g_signal_connect(typefind, "have-type", G_CALLBACK(gst_type_found_cb), 
-        &mimetype);
+    g_object_set(source, "location", uri, NULL);
+    g_signal_connect(typefind, "have-type", 
+        G_CALLBACK(gst_typefind_type_found_callback), &mimetype);
 
-    gst_bin_add_many(GST_BIN(pipeline), source, typefind, NULL);
+    gst_bin_add_many(GST_BIN(pipeline), source, typefind, fakesink, NULL);
     gst_element_link(source, typefind);
-    gst_element_set_state(GST_ELEMENT(pipeline), GST_STATE_PLAYING);
-
-    do {
-        if(!gst_bin_iterate(GST_BIN(pipeline))) {
-            break;
+    gst_element_link(typefind, fakesink);
+    
+    if(gst_element_set_state(pipeline, GST_STATE_PLAYING) == GST_STATE_CHANGE_ASYNC) {
+        if (gst_element_get_state(pipeline, NULL, NULL, -1) == GST_STATE_CHANGE_FAILURE) {
+            gst_object_unref(pipeline);
+            return NULL;
         }
-    } while(mimetype == NULL);
-
-    if(strcmp(mimetype, "error") == 0) {
-        mimetype = NULL;
     }
 
-    gst_element_set_state(GST_ELEMENT(pipeline), GST_STATE_NULL);
-    gst_object_unref(GST_OBJECT(pipeline));
-
-    return mimetype;
+    while((mimetype == NULL) && (pipeline->current_state == GST_STATE_PLAYING));
+    
+    gst_element_set_state(pipeline, GST_STATE_NULL);
+    gst_object_unref(pipeline);
+        
+    return mimetype;*/
 }
-
-
