@@ -1,8 +1,8 @@
 /***************************************************************************
  *  ActionManager.cs
  *
- *  Copyright (C) 2005 Novell
- *  Written by Aaron Bockover (aaron@aaronbock.net)
+ *  Copyright (C) 2005-2006 Novell, Inc.
+ *  Written by Aaron Bockover <aaron@abock.org>
  ****************************************************************************/
 
 /*  THIS FILE IS LICENSED UNDER THE MIT LICENSE AS OUTLINED IMMEDIATELY BELOW: 
@@ -43,13 +43,17 @@ namespace Banshee.Base
         private ActionGroup audio_cd_actions = new ActionGroup("AudioCD");
         private ActionGroup playback_actions = new ActionGroup("Playback");
         private ActionGroup playback_seek_actions = new ActionGroup("PlaybackSeek");
-        private ActionGroup source_eject_actions = new ActionGroup("SourceEject");
         private ActionGroup dap_actions = new ActionGroup("Dap");
         
         public ActionManager()
         {
             PopulateActionGroups();
+        }
+        
+        public void LoadInterface()
+        {
             ui.AddUiFromResource("UIManagerLayout.xml");
+            Gtk.Window.DefaultIconName = "music-player-banshee";
         }
         
         private void PopulateActionGroups()
@@ -73,13 +77,17 @@ namespace Banshee.Base
                     Catalog.GetString("Import files inside a folder"), null),
                     
                 new ActionEntry("ImportMusicAction", Stock.Open,
-                    Catalog.GetString("Import Music..."), null,
+                    Catalog.GetString("Import Music..."), "<control>I",
                     Catalog.GetString("Import music from a variety of sources"), null),
                     
                 new ActionEntry("OpenLocationAction", null, 
                     Catalog.GetString("Open Location..."), "<control>L",
                     Catalog.GetString("Open a remote location for playback"), null),
                     
+                new ActionEntry("WriteCDAction", null,
+                    Catalog.GetString("Write CD"), null,
+                    Catalog.GetString("Write selection to audio CD"), null),
+                    
                 new ActionEntry("ImportSourceAction", null,
                     Catalog.GetString("Import Source"), null,
                     Catalog.GetString("Import source to library"), null),
@@ -96,8 +104,12 @@ namespace Banshee.Base
                     Catalog.GetString("_Edit"), null, null, null),
                     
                 new ActionEntry("RenameSourceAction", "gtk-edit", 
-                    Catalog.GetString("Rename Source"), "F2",
-                    Catalog.GetString("Rename the active source"), null),
+                    "Rename", "F2",
+                    "Rename", null),
+
+                new ActionEntry("UnmapSourceAction", Stock.Delete,
+                    "Unmap", "<shift>Delete",
+                    null, null),
                     
                 new ActionEntry("SelectAllAction", null,
                     Catalog.GetString("Select All"), "<control>A",
@@ -106,6 +118,10 @@ namespace Banshee.Base
                 new ActionEntry("SelectNoneAction", null,
                     Catalog.GetString("Select None"), "<control><shift>A",
                     Catalog.GetString("Unselect all songs in song list"), null),
+
+                new ActionEntry("JumpToPlayingAction", null,
+                    Catalog.GetString("Jump to playing song"), "<control>J",
+                    null, null),
                 
                 new ActionEntry("PluginsAction", null,
                     Catalog.GetString("Plugins..."), null,
@@ -119,6 +135,10 @@ namespace Banshee.Base
                 new ActionEntry("ColumnsAction", null,
                     Catalog.GetString("Columns..."), null,
                     Catalog.GetString("Select which columns to display in the song list"), null),
+                
+                new ActionEntry("ShowEqualizerAction", null,
+                    Catalog.GetString("Equalizer"), null,
+                    Catalog.GetString("Display the equalizer."), null),
                     
                 new ActionEntry("LoggedEventsAction", null,
                     Catalog.GetString("Logged Events Viewer..."), null,
@@ -131,6 +151,27 @@ namespace Banshee.Base
                     Catalog.GetString("Version Information..."), null,
                     Catalog.GetString("View detailed version and configuration information"), null),
                     
+                new ActionEntry("WebMenuAction", null,
+                    Catalog.GetString("Web Resources"), null, null, null),
+                    
+                new ActionEntry("WikiGuideAction", Stock.Help,
+                    Catalog.GetString("Banshee User Guide (Wiki)"), null,
+                    Catalog.GetString("Learn about how to use Banshee"), delegate {
+                        Gnome.Url.Show("http://banshee-project.org/Guide");
+                    }),
+                    
+                new ActionEntry("WikiAction", null,
+                    Catalog.GetString("Banshee Home Page"), null,
+                    Catalog.GetString("Visit the Banshee Home Page"), delegate {
+                        Gnome.Url.Show("http://banshee-project.org/");
+                    }),
+                    
+                new ActionEntry("WikiDeveloperAction", null,
+                    Catalog.GetString("Get Involved"), null,
+                    Catalog.GetString("Become a contributor to Banshee"), delegate {
+                        Gnome.Url.Show("http://banshee-project.org/Developers");
+                    }),
+                    
                 new ActionEntry("AboutAction", "gtk-about", null),
                     
                 new ActionEntry("PlaybackMenuAction", null,
@@ -140,38 +181,29 @@ namespace Banshee.Base
                     Catalog.GetString("Source"), null, null, null),
                     
                 new ActionEntry("SongViewPopupAction", null, 
-                    "Song Menu", null, null, null)
+                    Catalog.GetString("Song Menu"), null, null, null),
+                    
+                new ActionEntry("DebugMenuAction", null,
+                    Catalog.GetString("Debug"), null, null, null)
             });
             
-            global_actions.Add(new ToggleActionEntry [] {
+            global_actions.Add(new ToggleActionEntry [] {               
                 new ToggleActionEntry("FullScreenAction", null,
                     Catalog.GetString("Fullscreen"), "F11",
                     Catalog.GetString("Toggle Fullscreen Mode"), null, false),
                 
                 new ToggleActionEntry("ShowCoverArtAction", null,
                     Catalog.GetString("Show Cover Art"), null,
-                    Catalog.GetString("Toggle display of album cover art"), null, false)
+                    Catalog.GetString("Toggle display of album cover art"), null, false),              
             });
 
+            global_actions.GetAction("ShowEqualizerAction").Visible = false;
+
             ui.InsertActionGroup(global_actions, 0);
 
-            /* Playlist Selected Actions */
-
-            playlist_actions.Add(new ActionEntry [] {
-                new ActionEntry("DeletePlaylistAction", Stock.Delete,
-                    Catalog.GetString("Delete Playlist"), "<shift>Delete",
-                    Catalog.GetString("Delete the active playlist"), null)
-            });
-            
-            ui.InsertActionGroup(playlist_actions, 0);
-
             /* Song Selected Actions */
             
             song_actions.Add(new ActionEntry [] {
-                new ActionEntry("WriteCDAction", null,
-                    Catalog.GetString("Write CD"), null,
-                    Catalog.GetString("Write selection to audio CD"), null),
-                    
                 new ActionEntry("RemoveSongsAction", Stock.Remove,
                     Catalog.GetString("Remove Song(s)"), "Delete",
                     Catalog.GetString("Remove selected song(s) from library"), null),
@@ -183,6 +215,22 @@ namespace Banshee.Base
                 new ActionEntry("PropertiesAction", Stock.Properties,
                     Catalog.GetString("Edit Song Metadata"), null,
                     Catalog.GetString("Edit metadata on selected songs"), null),
+
+                new ActionEntry("SearchMenuAction", Stock.Find,
+                    Catalog.GetString("Search for songs"), null,
+                    Catalog.GetString("Search for songs matching certain criteria"), null),
+
+                new ActionEntry("SearchForSameAlbumAction", null,
+                    Catalog.GetString("By matching album"), null,
+                    Catalog.GetString("Search all songs of this album"), null),
+
+                new ActionEntry("SearchForSameArtistAction", null,
+                    Catalog.GetString("By matching artist"), null,
+                    Catalog.GetString("Search all songs of this artist"), null),
+
+                new ActionEntry("SearchForSameGenreAction", null,
+                    Catalog.GetString("By matching genre"), null,
+                    Catalog.GetString("Search all songs of this genre"), null),
                     
                 new ActionEntry("AddToPlaylistAction", null,
                     Catalog.GetString("Add to Playlist"), null,
@@ -190,7 +238,7 @@ namespace Banshee.Base
                     
                 new ActionEntry("RatingAction", null,
                     Catalog.GetString("Rating"), null,
-                    Catalog.GetString("Set rating for selected songs"), null),
+                    Catalog.GetString("Set rating for selected songs"), null)
             });
             
             ui.InsertActionGroup(song_actions, 0);
@@ -238,7 +286,11 @@ namespace Banshee.Base
             playback_actions.Add(new ToggleActionEntry [] {
                 new ToggleActionEntry("ShuffleAction", "media-playlist-shuffle",
                     Catalog.GetString("Shuffle"), null,
-                    Catalog.GetString("Toggle between shuffle or continuous playback modes"), null, false)
+                    Catalog.GetString("Toggle between shuffle or continuous playback modes"), null, false),
+                    
+                new ToggleActionEntry("StopWhenFinishedAction", null,
+                    Catalog.GetString("Stop when finished"), "<Shift>space",
+                    Catalog.GetString("Stop playback after the current song finishes playing"), null, false)
             });
 
             ui.InsertActionGroup(playback_actions, 0);
@@ -275,15 +327,7 @@ namespace Banshee.Base
             
             ui.InsertActionGroup(dap_actions, 0);
             
-            /* Source Eject Actions */
-            
-            source_eject_actions.Add(new ActionEntry [] {
-                new ActionEntry("EjectSelectedSourceAction", "media-eject",
-                    Catalog.GetString("Eject"), null,
-                    Catalog.GetString("Eject the active source"), null)
-            });
-            
-            ui.InsertActionGroup(source_eject_actions, 0);
+            this["DebugMenuAction"].Visible = Globals.ArgumentQueue.Contains("debug");
         }
         
         public Action FindActionByName(string actionName)
@@ -371,10 +415,6 @@ namespace Banshee.Base
             get { return playback_seek_actions; }
         }
         
-        public ActionGroup SourceEjectActions {
-            get { return source_eject_actions; }
-        }
-        
         public ActionGroup DapActions {
             get { return dap_actions; }
         }
