@@ -52,7 +52,7 @@ namespace Banshee.Base
     {
         public AudioCdTrackInfo Track;
         public int TrackNumber;
-        public Uri Uri;
+        public SafeUri Uri;
     }
 
     public class AudioCdTrackRipper : IDisposable
@@ -64,13 +64,11 @@ namespace Banshee.Base
         private HandleRef handle;
         private GstCdRipperProgressCallback progress_callback;
 
-#if GSTREAMER_0_10
         private GstCdRipperFinishedCallback finished_callback;
         private GstCdRipperErrorCallback error_callback;
         private string error_message;
-        private Uri output_uri;
+        private SafeUri output_uri;
         private int track_number;
-#endif
 
         private AudioCdTrackInfo current_track;
         
@@ -90,13 +88,11 @@ namespace Banshee.Base
             progress_callback = new GstCdRipperProgressCallback(OnProgress);
             gst_cd_ripper_set_progress_callback(handle, progress_callback);
 
-#if GSTREAMER_0_10
             finished_callback = new GstCdRipperFinishedCallback(OnFinished);
             gst_cd_ripper_set_finished_callback(handle, finished_callback);
             
             error_callback = new GstCdRipperErrorCallback(OnError);
             gst_cd_ripper_set_error_callback(handle, error_callback);
-#endif
         }
         
         public void Dispose()
@@ -104,20 +100,13 @@ namespace Banshee.Base
             gst_cd_ripper_free(handle);
         }
         
-        public void RipTrack(AudioCdTrackInfo track, int trackNumber, Uri outputUri)
+        public void RipTrack(AudioCdTrackInfo track, int trackNumber, SafeUri outputUri)
         {
-#if GSTREAMER_0_10
             error_message = null;
             RipTrack_010(track, trackNumber, outputUri);
-#else
-            ThreadAssist.Spawn(delegate {
-                RipTrack_08(track, trackNumber, outputUri);
-            });
-#endif
         }
   
-#if GSTREAMER_0_10
-        private void RipTrack_010(AudioCdTrackInfo track, int trackNumber, Uri outputUri)
+        private void RipTrack_010(AudioCdTrackInfo track, int trackNumber, SafeUri outputUri)
         {
             current_track = track;
             track_number = trackNumber;
@@ -129,26 +118,8 @@ namespace Banshee.Base
                 
             track = null;
         }
-#else        
-        private void RipTrack_08(AudioCdTrackInfo track, int trackNumber, Uri outputUri)
-        {
-            current_track = track;
-            
-            bool result = gst_cd_ripper_rip_track(handle, outputUri.AbsoluteUri, trackNumber, 
-                track.Artist, track.Album, track.Title, track.Genre, 
-                (int)track.TrackNumber, (int)track.TrackCount, IntPtr.Zero);
-                
-            if(result) {
-                ThreadAssist.ProxyToMain(delegate {
-                    OnTrackFinished(track, trackNumber, outputUri);
-                });
-            }
-            
-            track = null;
-        }
-#endif   
-        
-        private void OnTrackFinished(AudioCdTrackInfo track, int trackNumber, Uri outputUri)
+       
+        private void OnTrackFinished(AudioCdTrackInfo track, int trackNumber, SafeUri outputUri)
         {
             track.IsRipped = true;
             track.Uri = outputUri;
@@ -170,16 +141,7 @@ namespace Banshee.Base
         
         public string ErrorMessage {
             get {
-#if GSTREAMER_0_10
                 return error_message;
-#else
-                IntPtr errPtr = gst_cd_ripper_get_error(handle);
-                if(errPtr == IntPtr.Zero) {
-                    return null;
-                }
-                
-                return GLib.Marshaller.Utf8PtrToString(errPtr);
-#endif
             }
         }
         
@@ -196,8 +158,7 @@ namespace Banshee.Base
             
             handler(this, args); 
         }
-        
-#if GSTREAMER_0_10        
+ 
         private void OnFinished(IntPtr ripper)
         {
             OnTrackFinished(current_track, track_number, output_uri);
@@ -218,7 +179,6 @@ namespace Banshee.Base
                 Error(this, new EventArgs());
             }
         }
-#endif
 
         [DllImport("libbanshee")]
         private static extern IntPtr gst_cd_ripper_new(string device,
@@ -240,7 +200,6 @@ namespace Banshee.Base
         [DllImport("libbanshee")]
         private static extern void gst_cd_ripper_cancel(HandleRef ripper);
                     
-#if GSTREAMER_0_10 
         [DllImport("libbanshee")]
         private static extern void gst_cd_ripper_set_error_callback(
             HandleRef ripper, GstCdRipperErrorCallback cb);
@@ -248,10 +207,6 @@ namespace Banshee.Base
         [DllImport("libbanshee")]
         private static extern void gst_cd_ripper_set_finished_callback(
             HandleRef ripper, GstCdRipperFinishedCallback cb);
-#else
-        [DllImport("libbanshee")]
-        private static extern IntPtr gst_cd_ripper_get_error(HandleRef ripper);
-#endif
     }
 
     public class AudioCdRipper
@@ -279,49 +234,21 @@ namespace Banshee.Base
         
         private ActiveUserEvent user_event;
         
-        [DllImport("libc")]
-        private static extern int ioctl(int device, IoctlOperation request, bool lockdoor); 
-
-        private enum IoctlOperation {
-            LockDoor = 0x5329
-        }
-        
-        private static bool LockDrive(string device, bool lockdoor)
-        {
-            using(UnixStream stream = (new UnixFileInfo(device)).Open( 
-                Mono.Unix.Native.OpenFlags.O_RDONLY | 
-                Mono.Unix.Native.OpenFlags.O_NONBLOCK)) {
-                return ioctl(stream.Handle, IoctlOperation.LockDoor, lockdoor) == 0;
-            }
-        }
-        
-        private void LockDrive()
-        {
-            lock(this) {
-                if(!LockDrive(device, true)) {
-                    LogCore.Instance.PushWarning("Could not lock CD-ROM drive", device, false);
-                }
-            }
-        }
-        
-        private void UnlockDrive()
-        {
-            lock(this) {
-                if(!LockDrive(device, false)) {
-                    LogCore.Instance.PushWarning("Could not unlock CD-ROM drive", device, false);
-                }
-            }
-        }
-        
         public AudioCdRipper()
         {
-            user_event = new ActiveUserEvent("Importing CD");
+            user_event = new ActiveUserEvent(Catalog.GetString("Importing CD"));
             user_event.Icon = IconThemeUtils.LoadIcon("cd-action-rip-24", 22);
             user_event.CancelRequested += OnCancelRequested;
         }
 
         public void QueueTrack(AudioCdTrackInfo track)
         {
+            if(user_event.CancelMessage == null) {
+                user_event.CancelMessage = String.Format(Catalog.GetString(
+                    "<i>{0}</i> is still being imported into the music library. Would you like to stop it?"
+                ), GLib.Markup.EscapeText(track.Album));
+            }
+            
             if(device == null) {
                 device = track.Device;
             } else if(device != track.Device) {
@@ -349,8 +276,6 @@ namespace Banshee.Base
         
                 LogCore.Instance.PushDebug("Ripping CD and Encoding with Pipeline", encodePipeline);
             
-                LockDrive();
-            
                 ripper = new AudioCdTrackRipper(device, 0, encodePipeline);
                 ripper.Progress += OnRipperProgress;
                 ripper.TrackFinished += OnTrackRipped;
@@ -366,12 +291,6 @@ namespace Banshee.Base
             }
         }
         
-        public void Cancel()
-        {
-            user_event.Dispose();
-            OnFinished();
-        }
-        
         private void RipNextTrack()
         {
             if(tracks.Count <= 0) {
@@ -384,7 +303,7 @@ namespace Banshee.Base
             status = String.Format("{0} - {1}", track.Artist, track.Title);
             user_event.Message = status;
 
-            Uri uri = PathUtil.PathToFileUri(FileNamePattern.BuildFull(track, profile.Extension));
+            SafeUri uri = new SafeUri(FileNamePattern.BuildFull(track, profile.Extension));
 
             ripper.RipTrack(track, track.TrackIndex, uri);
         }
@@ -437,7 +356,6 @@ namespace Banshee.Base
         
         private void OnFinished()
         {
-            UnlockDrive();
             EventHandler handler = Finished;
             if(handler != null) {
                 handler(this, new EventArgs());
