@@ -40,8 +40,8 @@ namespace Banshee.Base
     public class FileCompleteArgs : EventArgs
     {
         public TrackInfo Track;
-        public Uri InputUri;
-        public Uri EncodedFileUri;
+        public SafeUri InputUri;
+        public SafeUri EncodedFileUri;
     }
 
     public class BatchTranscoder
@@ -49,9 +49,9 @@ namespace Banshee.Base
         public class QueueItem
         {
             private object source;
-            private Uri destination;
+            private SafeUri destination;
             
-            public QueueItem(object source, Uri destination)
+            public QueueItem(object source, SafeUri destination)
             {
                 this.source = source;
                 this.destination = destination;
@@ -61,7 +61,7 @@ namespace Banshee.Base
                 get { return source; }
             }
             
-            public Uri Destination {
+            public SafeUri Destination {
                 get { return destination; }
             }
         }
@@ -75,6 +75,7 @@ namespace Banshee.Base
         private PipelineProfile profile;
         private int finished_count;
         private int total_count;
+        private string desired_profile_name;
         
         private ActiveUserEvent user_event;
         
@@ -84,22 +85,27 @@ namespace Banshee.Base
         
         private const int progress_precision = 1000;
         
-        public BatchTranscoder(PipelineProfile profile)
+        public BatchTranscoder(PipelineProfile profile) : this(profile, null)
+        {
+        }
+        
+        public BatchTranscoder(PipelineProfile profile, string desiredProfileName)
         {
             transcoder = new GstTranscoder();
             transcoder.Progress += OnTranscoderProgress;
             transcoder.Error += OnTranscoderError;
             transcoder.Finished += OnTranscoderFinished;
             
+            this.desired_profile_name = desiredProfileName;
             this.profile = profile;
         }
         
-        public void AddTrack(TrackInfo track, Uri outputUri)
+        public void AddTrack(TrackInfo track, SafeUri outputUri)
         {
             batch_queue.Enqueue(new QueueItem(track, outputUri));
         }
         
-        public void AddTrack(Uri inputUri, Uri outputUri)
+        public void AddTrack(SafeUri inputUri, SafeUri outputUri)
         {
             batch_queue.Enqueue(new QueueItem(inputUri, outputUri));
         }
@@ -107,8 +113,10 @@ namespace Banshee.Base
         public void Start()
         {
             if(user_event == null) {
-                user_event = new ActiveUserEvent(Catalog.GetString("File Transcoder"));
-                user_event.Header = Catalog.GetString("Transcoding Files");
+                user_event = new ActiveUserEvent(Catalog.GetString("Converting Files"));
+                user_event.Header = Catalog.GetString("Converting Files");
+                user_event.CancelMessage = Catalog.GetString(
+                    "Files are currently being converted to another audio format. Would you like to stop this?");
                 user_event.CancelRequested += OnCancelRequested;
                 user_event.Icon = IconThemeUtils.LoadIcon("encode-action-24", 22);
                 user_event.Message = Catalog.GetString("Initializing Transcoder...");
@@ -128,15 +136,15 @@ namespace Banshee.Base
                 return;
             }
             
-            Uri output_uri = current.Destination;
-            Uri input_uri = null;
+            SafeUri output_uri = current.Destination;
+            SafeUri input_uri = null;
             
             if(current.Source is TrackInfo) {
                 TrackInfo track = current.Source as TrackInfo;
                 user_event.Message = String.Format("{0} - {1}", track.DisplayArtist, track.DisplayTitle);
                 input_uri = track.Uri;
-            } else if(current.Source is Uri) {
-                input_uri = current.Source as Uri;
+            } else if(current.Source is SafeUri) {
+                input_uri = current.Source as SafeUri;
                 user_event.Message = Path.GetFileName(input_uri.LocalPath);
             } else {
                 return;
@@ -148,6 +156,8 @@ namespace Banshee.Base
             
             if(Path.GetExtension(input_uri.LocalPath) != "." + profile.Extension) {
                 transcoder.BeginTranscode(input_uri, output_uri, profile);
+            } else if(desired_profile_name != null && profile.Name != desired_profile_name) {
+                OnTranscoderError(this, new EventArgs());
             } else {
                 OnTranscoderFinished(this, new EventArgs());
             }   
@@ -181,8 +191,8 @@ namespace Banshee.Base
                 if(current.Source is TrackInfo) {
                     cargs.Track = current.Source as TrackInfo;
                     cargs.InputUri = cargs.Track.Uri;
-                } else if(current.Source is Uri) {
-                    cargs.InputUri = (current.Source as Uri);
+                } else if(current.Source is SafeUri) {
+                    cargs.InputUri = (current.Source as SafeUri);
                 }
                 
                 handler(this, cargs);
