@@ -29,15 +29,18 @@
 using System;
 using System.IO;
 using System.Collections;
+using System.Collections.Generic;
 using Mono.Unix;
 
+using Banshee.Gui;
 using Banshee.Base;
 using Banshee.Widgets;
 using Banshee.Sources;
+using Banshee.Plugins;
 
 namespace Banshee.Dap
 {    
-    public abstract class DapDevice : IEnumerable, IDisposable
+    public abstract class DapDevice : IPlugin, IEnumerable<TrackInfo>, IEnumerable
     {
         public class Property
         {
@@ -123,11 +126,12 @@ namespace Banshee.Dap
         
         private uint uid;
         private PropertyTable properties = new PropertyTable();
-        private ArrayList tracks = new ArrayList(); 
         private ActiveUserEvent save_report_event;
         private bool is_syncing = false;
         private bool can_cancel_save = true;
         private DapProperties type_properties;
+
+        protected List<TrackInfo> tracks = new List<TrackInfo>(); 
         
         public event DapTrackListUpdatedHandler TrackAdded;
         public event DapTrackListUpdatedHandler TrackRemoved;
@@ -162,7 +166,12 @@ namespace Banshee.Dap
             }
         }
         
-        public IEnumerator GetEnumerator()
+        IEnumerator IEnumerable.GetEnumerator()
+        {
+            return tracks.GetEnumerator();
+        }
+        
+        public IEnumerator<TrackInfo> GetEnumerator()
         {
             return tracks.GetEnumerator();
         }
@@ -188,20 +197,14 @@ namespace Banshee.Dap
             OnPropertiesChanged();
         }
         
-        public void AddTrack(TrackInfo track)
+        public virtual void AddTrack(TrackInfo track)
         {
-            TrackInfo dap_track = OnTrackAdded(track);
-            
-            if(dap_track == null) {
+            if (track == null)
                 return;
-            }
             
-            tracks.Add(dap_track);
+            tracks.Add(track);
             
-            DapTrackListUpdatedHandler handler = TrackAdded;
-            if(handler != null) {
-                handler(this, new DapTrackListUpdatedArgs(dap_track));
-            }
+            OnTrackAdded(track);
         }
         
         public void RemoveTrack(TrackInfo track)
@@ -237,9 +240,12 @@ namespace Banshee.Dap
             }
         }
         
-        protected virtual TrackInfo OnTrackAdded(TrackInfo track)
+        protected virtual void OnTrackAdded (TrackInfo track)
         {
-            return track;
+            DapTrackListUpdatedHandler handler = TrackAdded;
+            if(handler != null) {
+                handler(this, new DapTrackListUpdatedArgs(track));
+            }
         }
         
         protected virtual void OnTrackRemoved(TrackInfo track)
@@ -276,7 +282,7 @@ namespace Banshee.Dap
                 a.TrackNumber == b.TrackNumber;
         }
         
-        protected bool TrackExistsInList(TrackInfo track, ICollection list)
+        protected bool TrackExistsInList(TrackInfo track, ICollection<TrackInfo> list)
         {
             try {
                 foreach(TrackInfo track_b in list) {
@@ -290,7 +296,7 @@ namespace Banshee.Dap
             return false;
         }
 
-        public void Save(ICollection library)
+        public void Save(ICollection<TrackInfo> library)
         {
             Queue remove_queue = new Queue();
             
@@ -326,8 +332,11 @@ namespace Banshee.Dap
                 handler(this, new EventArgs());
             }
             
-            save_report_event = new ActiveUserEvent(Catalog.GetString("Synchronizing Device"));
-            save_report_event.Header = Catalog.GetString("Synchronizing Device");
+            string aue_name = Name == null || Name == String.Empty ? Catalog.GetString("Device") : Name; 
+            string aue_title = String.Format(Catalog.GetString("Synchronizing {0}"), aue_name); 
+                
+            save_report_event = new ActiveUserEvent(aue_title);
+            save_report_event.Header = aue_title;
             save_report_event.Message = Catalog.GetString("Waiting for transcoder...");
             save_report_event.Icon = GetIcon(22);
             save_report_event.CanCancel = can_cancel_save;
@@ -336,15 +345,11 @@ namespace Banshee.Dap
         }
         
         protected bool ShouldCancelSave {
-            get {
-                return save_report_event.IsCancelRequested;
-            }
+            get { return save_report_event.IsCancelRequested; }
         }
         
         protected bool CanCancelSave {
-            set {
-                can_cancel_save = value;
-            }
+            set { can_cancel_save = value; }
         }
         
         protected void UpdateSaveProgress(string header, string message, double progress)
@@ -391,25 +396,24 @@ namespace Banshee.Dap
                         continue;
                     }
 
-                    Uri old_uri = track.Uri;
-                    
+                    SafeUri old_uri = track.Uri;
                     try {
                         track.Uri = ConvertSongFileName(track.Uri, profile.Extension);
                     } catch {
                         continue;
                     }
-                    
+
                     if(encoder == null) {
-                        encoder = new BatchTranscoder(profile);
+                        encoder = new BatchTranscoder(profile, type_properties.PipelineName);
                         encoder.FileFinished += OnFileEncodeComplete;
                         encoder.BatchFinished += OnFileEncodeBatchFinished;
                         encoder.Canceled += OnFileEncodeCanceled;
                     }
-                        
+                    
                     encoder.AddTrack(old_uri, track.Uri);
                 } else {
                     if(System.IO.File.Exists(cached_filename)) {
-                        track.Uri = PathUtil.PathToFileUri(cached_filename);
+                        track.Uri = new SafeUri(cached_filename);
                     } else {
                         remove_queue.Enqueue(track);
                     }
@@ -443,16 +447,65 @@ namespace Banshee.Dap
         {
             if(!encoder_canceled) {
                 save_report_event.Message = Catalog.GetString("Processing...");
-                if(ThreadAssist.InMainThread) {
-                    ThreadAssist.Spawn(Synchronize);
+                
+                BatchTranscoder encoder = o as BatchTranscoder;
+                if(encoder.ErrorCount > 0) {
+                    ThreadAssist.ProxyToMain(delegate {
+                        HandleTranscodeErrors(encoder);
+                    });
                 } else {
-                    Synchronize();
+                    ChainSynchronize();
                 }
             } else {
                 FinishSave();
             }
         }
         
+        private void HandleTranscodeErrors(BatchTranscoder encoder)
+        {
+            ErrorListDialog dialog = new ErrorListDialog();
+            dialog.IconNameStock = Gtk.Stock.DialogError;
+            dialog.Header = Catalog.GetString("Could not encode some files");
+            dialog.Message = Catalog.GetString(
+                "Some files could not be encoded to the proper format. " +
+                "They will not be saved to the device if you continue."
+            );
+            
+            dialog.AddStockButton(Gtk.Stock.Cancel, Gtk.ResponseType.Cancel);
+            dialog.AddButton(Catalog.GetString("Continue synchronizing"), Gtk.ResponseType.Ok);
+            
+            foreach(BatchTranscoder.QueueItem item in encoder.ErrorList) {
+                if(item.Source is TrackInfo) {
+                    TrackInfo track = item.Source as TrackInfo;
+                    dialog.AppendString(String.Format("{0} - {1}", track.Artist, track.Title));
+                } else if(item.Source is SafeUri) {
+                    SafeUri uri = item.Source as SafeUri;
+                    dialog.AppendString(System.IO.Path.GetFileName(uri.LocalPath));
+                } else {
+                    dialog.AppendString(item.Source.ToString());
+                }
+            }
+            
+            try {
+                if(dialog.Run() == Gtk.ResponseType.Ok) {
+                    ChainSynchronize();
+                } else {
+                    FinishSave();
+                }
+            } finally {
+                dialog.Destroy();
+            }   
+        }
+        
+        private void ChainSynchronize()
+        {
+            if(ThreadAssist.InMainThread) {
+                ThreadAssist.Spawn(Synchronize);
+            } else {
+                Synchronize();
+            }
+        }
+        
         private string [] supported_extensions = null;
         protected string [] SupportedExtensions {
             get {        
@@ -497,7 +550,7 @@ namespace Banshee.Dap
             return false;
         }
         
-        private string GetCachedSongFileName(Uri uri)
+        private string GetCachedSongFileName(SafeUri uri)
         {
             string filename = uri.LocalPath;
         
@@ -528,7 +581,7 @@ namespace Banshee.Dap
             return null;
         }
         
-        private Uri ConvertSongFileName(Uri uri, string newext)
+        private SafeUri ConvertSongFileName(SafeUri uri, string newext)
         {
             string filename = uri.LocalPath;
             
@@ -536,10 +589,10 @@ namespace Banshee.Dap
             string dir = Path.GetDirectoryName(path);
             string file = Path.GetFileNameWithoutExtension(filename);
             
-            return PathUtil.PathToFileUri(dir + Path.DirectorySeparatorChar 
+            return new SafeUri(dir + Path.DirectorySeparatorChar 
                 + ".banshee-dap-" + file + "." + newext);
         }
-         
+
         public virtual Gdk.Pixbuf GetIcon(int size)
         {
             Gdk.Pixbuf pixbuf = IconThemeUtils.LoadIcon("multimedia-player", size);
@@ -559,57 +612,39 @@ namespace Banshee.Dap
         }
         
         public PropertyTable Properties {
-            get {
-                return properties;
-            }
+            get { return properties; }
         }
         
         public TrackInfo this [int index] {
-            get {
-                return tracks[index] as TrackInfo;
-            }
+            get { return tracks[index]; }
         }
         
         public int TrackCount { 
-            get {
-                return tracks.Count;
-            }
+            get { return tracks.Count; }
         }
         
-        public IList Tracks {
-            get {
-                return tracks;
-            }
+        public IList<TrackInfo> Tracks {
+            get { return tracks; }
         }
         
         public bool IsSyncing {
-            get {
-                return is_syncing;
-            }
+            get { return is_syncing; }
         }
         
         public virtual string GenericName {
-            get {
-                return "DAP";
-            }
+            get { return "DAP"; }
         }
         
         public string HalUdi {
-            get {
-                return HalDevice.Udi;
-            }
+            get { return HalDevice.Udi; }
         }
         
         public bool CanSetName {
-            get {
-                return ReflectionUtil.IsVirtualMethodImplemented(GetType(), "SetName");
-            }
+            get { return ReflectionUtil.IsVirtualMethodImplemented(GetType(), "SetName"); }
         }
         
         public bool CanSetOwner {
-            get {
-                return ReflectionUtil.IsVirtualMethodImplemented(GetType(), "SetOwner");
-            }
+            get { return ReflectionUtil.IsVirtualMethodImplemented(GetType(), "SetOwner"); }
         }
         
         public virtual bool CanSynchronize { 
@@ -617,15 +652,11 @@ namespace Banshee.Dap
         }
         
         public virtual string Owner {
-            get {
-                return Catalog.GetString("Unknown");
-            }
+            get { return Catalog.GetString("Unknown"); }
         }
         
         public virtual Gtk.Widget ViewWidget {
-            get {
-                return null;
-            }
+            get { return null; }
         }
         
         public abstract void Synchronize();
