@@ -29,9 +29,12 @@
 using System;
 using System.IO;
 using GConf;
+using Mono.Unix;
 
 namespace Banshee.Base
 {
+    public delegate bool ShutdownRequestHandler();
+
     public static class Globals
     {
         private static GConf.Client gconf_client;
@@ -42,7 +45,11 @@ namespace Banshee.Base
         private static AudioCdCore audio_cd_core;
         private static Random random;
         private static DBusRemote dbus_remote;
+        private static DBusPlayer dbus_player;
         private static Banshee.Gui.UIManager ui_manager;
+        private static ComponentInitializer startup = new ComponentInitializer();
+        
+        public static event ShutdownRequestHandler ShutdownRequested;
         
         public static void Initialize()
         {
@@ -53,99 +60,134 @@ namespace Banshee.Base
             Mono.Unix.Catalog.Init(ConfigureDefines.GETTEXT_PACKAGE, ConfigureDefines.LOCALE_DIR);
         
             ui_manager = new Banshee.Gui.UIManager();
+            random = new Random();
             
             if(!Branding.Initialize()) {
                 System.Environment.Exit(1);
                 return;
             }
-        
-            gconf_client = new GConf.Client();
-            network_detect = NetworkDetect.Instance;
-            action_manager = new ActionManager();
-            library = new Library();
-            random = new Random();
+                        
+            startup.Register(Catalog.GetString("Starting background tasks"), delegate {
+                try {
+                    dbus_remote = new DBusRemote();
+                    dbus_player = new DBusPlayer();
+                    dbus_remote.RegisterObject(dbus_player, "Player");
+                } catch {
+                }
+            });
             
-            Banshee.Gstreamer.Utilities.Initialize();
-            PlayerEngineCore.Initialize();
+            startup.Register(Catalog.GetString("Starting background tasks"), delegate { gconf_client = new GConf.Client(); });
+            startup.Register(Catalog.GetString("Detecting network settings"), delegate { network_detect = NetworkDetect.Instance; });
+            startup.Register(Catalog.GetString("Creating action manager"), delegate { action_manager = new ActionManager(); });
+            startup.Register(Catalog.GetString("Loading music library"), delegate { 
+                library = new Library(); 
+                library.ReloadLibrary();
+            });
             
-            try {
-                audio_cd_core = new AudioCdCore();
-            } catch(ApplicationException e) {
-                LogCore.Instance.PushWarning("Audio CD support will be disabled for this instance", e.Message, false);
-            }
+            startup.Register(Catalog.GetString("Initializing player engine"), Banshee.Gstreamer.Utilities.Initialize);
+            startup.Register(Catalog.GetString("Initializing player engine"), PlayerEngineCore.Initialize);
             
-            try {
-                Banshee.Dap.DapCore.Initialize();
-            } catch(ApplicationException e) {
-                LogCore.Instance.PushWarning("DAP support will be disabled for this instance", e.Message, false);
-            }
+            startup.Register(Catalog.GetString("Initializing audio CD support"), true, 
+                Catalog.GetString("Audio CD support will be disabled for this instance"), delegate { audio_cd_core = new AudioCdCore(); });
+                
+            startup.Register(Catalog.GetString("Initializing digital audio player support"), true, 
+                Catalog.GetString("DAP support will be disabled for this instance"), Banshee.Dap.DapCore.Initialize);
             
-            Banshee.Plugins.PluginCore.Initialize();
+            startup.Register(Catalog.GetString("Initializing CD writing support"), true, 
+                Catalog.GetString("CD burning support will be disabled for this instance"), Banshee.Burner.BurnerCore.Initialize);
+                
+            startup.Register(Catalog.GetString("Initializing plugins"), Banshee.Plugins.PluginCore.Initialize);
+            startup.Register(Catalog.GetString("Initializing background tasks"), PowerManagement.Initialize);
+            
+            startup.Run();
+            
+            action_manager.LoadInterface();
         }
         
-        public static void Dispose()
+        public static void Shutdown()
         {
+            if(Banshee.Kernel.Scheduler.IsScheduled(typeof(Banshee.Kernel.IInstanceCriticalJob)) ||
+                Banshee.Kernel.Scheduler.CurrentJob is Banshee.Kernel.IInstanceCriticalJob) {
+                Banshee.Gui.Dialogs.ConfirmShutdownDialog dialog = new Banshee.Gui.Dialogs.ConfirmShutdownDialog();
+                try {
+                    if(dialog.Run() == Gtk.ResponseType.Cancel) {
+                        return;
+                    }
+                } finally {
+                    dialog.Destroy();
+                }
+            }
+            
+            if(OnShutdownRequested()) {
+                Dispose();
+            }
+        }
+        
+        private static bool OnShutdownRequested()
+        {
+            ShutdownRequestHandler handler = ShutdownRequested;
+            if(handler != null) {
+                foreach(Delegate d in handler.GetInvocationList()) {
+                    if(!(bool)d.DynamicInvoke(null)) {
+                        return false;
+                    }
+                }
+            }
+            
+            return true;
+        }
+        
+        private static void Dispose()
+        {
+            dbus_remote.UnregisterObject(dbus_player);
+            Banshee.Kernel.Scheduler.Dispose();
             Banshee.Plugins.PluginCore.Dispose();
             network_detect.Dispose();
             library.Db.Dispose();
             Banshee.Dap.DapCore.Dispose();
             HalCore.Dispose();
+            PowerManagement.Dispose();
+        }
+        
+        public static ComponentInitializer StartupInitializer {
+            get { return startup; }
         }
         
         public static GConf.Client Configuration {
-            get {
-                return gconf_client;
-            }
+            get { return gconf_client; }
         }
 
         public static NetworkDetect Network {
-            get {
-                return network_detect;
-            }
+            get { return network_detect; }
         }
         
         public static ActionManager ActionManager {
-            get {
-                return action_manager;
-            }
+            get { return action_manager; }
         }
         
         public static Library Library {
-            get {
-                return library;
-            }
+            get { return library; }
         }
         
         public static ArgumentQueue ArgumentQueue {
-            set {
-                argument_queue = value;
-            }
-            
-            get {
-                return argument_queue;
-            }
+            set { argument_queue = value; }
+            get { return argument_queue; }
         }
         
         public static AudioCdCore AudioCdCore {
-            get {
-                return audio_cd_core;
-            }
+            get { return audio_cd_core; }
         }
         
         public static Random Random {
-            get {
-                return random;
-            }
+            get { return random; }
         }
         
         public static DBusRemote DBusRemote {
-            get {
-                return dbus_remote;
-            }
-            
-            set {
-                dbus_remote = value;
-            }
+            get { return dbus_remote; }
+        }
+        
+        public static DBusPlayer DBusPlayer {
+            get { return dbus_player; }
         }
         
         public static Banshee.Gui.UIManager UIManager {
@@ -185,8 +227,8 @@ namespace Banshee.Base
             }
         }
 	
-        private static Gtk.TreeView playlist_view;
-        public static Gtk.TreeView PlaylistView {
+        private static Gtk.Widget playlist_view;
+        public static Gtk.Widget PlaylistView {
             get { return playlist_view; }
             set {
                 if(playlist_view == null) {
@@ -205,9 +247,19 @@ namespace Banshee.Base
             }
         }
         
+        private static Gtk.Box action_button_box;
+        public static Gtk.Box ActionButtonBox {
+            get { return action_button_box; }
+            set {
+                if(action_button_box == null) {
+                    action_button_box = value;
+                }
+            }
+        }        
+        
         public static void DetachPlaylistContainer()
         {
-            if(PlaylistContainer.Parent != null) {
+            if(PlaylistContainer != null && PlaylistContainer.Parent != null) {
                 (PlaylistContainer.Parent as Gtk.Container).Remove(PlaylistContainer);
             }
         }
