@@ -68,7 +68,7 @@ namespace Banshee.Base
             "ogg |Ogg Vorbis|ogg |vorbisenc|bitrate|true |none",
             "flac|Flac      |flac|flacenc  |none   |false|quality=6",
             "wave|Wave / PCM|wav |wavenc   |none   |false|none",
-            "faac|Faac MP4  |mp4 |faac     |bitrate|false|none"
+            "faac|Faac MP4  |mp4 |faac     |bitrate|true |outputformat=1"
         };
         
         private static PipelineProfile [] loadedProfiles = null;
@@ -162,6 +162,10 @@ namespace Banshee.Base
         
         private void TestPipeline(string pipeline)
         {
+            if(pipeline == "cdwavenc") {
+                pipeline = "wavenc";
+            }
+            
             if(!Banshee.Gstreamer.Utilities.TestEncoder(pipeline)) {
                throw new PipelineProfileException(String.Format(
                    Catalog.GetString("Pipeline profile '{0}' will be " +
@@ -190,6 +194,10 @@ namespace Banshee.Base
         public string Pipeline
         {
             get {
+                if(element == "cdwavenc") {
+                    return element;
+                }
+            
                 if(!useInternalBitrate)
                     throw new PipelineProfileException(Catalog.GetString(
                         "Cannot use internal bitrate. Use " + 
@@ -353,6 +361,8 @@ namespace Banshee.Base
             96,
             48
         };
+        
+        public event EventHandler Changed;
        
         public PipelineProfileSelector(string extFilter)
         {
@@ -401,7 +411,7 @@ namespace Banshee.Base
             } else {
                 Label label = new Label();
                 label.Markup = "<i><small>" + 
-                    Catalog.GetString("No iPod-compatible encoders available") +
+                    Catalog.GetString("No MP3-compatible encoders available") +
                     "</small></i>";
                 PackStart(label, false, false, 0);
                 label.Show();
@@ -420,11 +430,20 @@ namespace Banshee.Base
         private void OnProfileChanged(object o, EventArgs args)
         {
             SetBitrateVisibility();
+            OnChanged();
         }
         
         private void OnBitrateChanged(object o, EventArgs args)
         {
+            OnChanged();
+        }
         
+        private void OnChanged()
+        {
+            EventHandler handler = Changed;
+            if(handler != null) {
+                handler(this, new EventArgs());
+            }
         }
         
         private void SetBitrateVisibility()
