@@ -1,8 +1,8 @@
 /***************************************************************************
  *  SourceManager.cs
  *
- *  Copyright (C) 2005 Novell
- *  Written by Aaron Bockover (aaron@aaronbock.net)
+ *  Copyright (C) 2005-2006 Novell, Inc.
+ *  Written by Aaron Bockover <aaron@abock.org>
  ****************************************************************************/
 
 /*  THIS FILE IS LICENSED UNDER THE MIT LICENSE AS OUTLINED IMMEDIATELY BELOW: 
@@ -61,7 +61,7 @@ namespace Banshee.Sources
         
         public static void AddSource(Source source, bool isDefault)
         {
-            if(source == null) {
+            if(source == null || ContainsSource (source)) {
                 return;
             }
             
@@ -76,7 +76,9 @@ namespace Banshee.Sources
             source.ViewChanged += OnSourceViewChanged;
             source.TrackAdded += OnSourceTrackAdded;
             source.TrackRemoved += OnSourceTrackRemoved;
-
+            source.ChildSourceAdded += OnChildSourceAdded;
+            source.ChildSourceRemoved += OnChildSourceRemoved;
+            
             ThreadAssist.ProxyToMain(delegate {
                 SourceAddedHandler handler = SourceAdded;
                 if(handler != null) {
@@ -86,6 +88,10 @@ namespace Banshee.Sources
                     handler(args);
                 }
             });
+            
+            foreach(Source child_source in source.Children) {
+                AddSource(child_source, false);
+            }
         }
         
         public static void AddSource(Source source)
@@ -160,6 +166,16 @@ namespace Banshee.Sources
                 handler(o, args);
             }
         }
+
+        private static void OnChildSourceAdded(SourceEventArgs args)
+        {
+            args.Source.Updated += OnSourceUpdated;
+        }
+        
+        private static void OnChildSourceRemoved(SourceEventArgs args)
+        {
+            args.Source.Updated -= OnSourceUpdated;
+        }
         
         private static int FindSourceInsertPosition(Source source)
         {
@@ -179,19 +195,12 @@ namespace Banshee.Sources
         }
         
         public static Source DefaultSource {
-            get {
-                return default_source;
-            }
-            
-            set {
-                default_source = value;
-            }
+            get { return default_source; }
+            set { default_source = value; }
         }
         
         public static Source ActiveSource {
-            get {
-                return active_source;
-            }
+            get { return active_source; }
         }
         
         public static void SetActiveSource(Source source)
@@ -201,8 +210,9 @@ namespace Banshee.Sources
         
         public static void SetActiveSource(Source source, bool notify)
         {
-            if (active_source != null)
-                active_source.Deactivate ();
+            if(active_source != null) {
+                active_source.Deactivate();
+            }
             
             active_source = source;
             source.Activate();
@@ -221,22 +231,8 @@ namespace Banshee.Sources
             });
         }
         
-        public static int ActiveSourceIndex {
-            get {
-                for(int i = 0; i < sources.Count; i++) {
-                    if((sources[i] as Source) == active_source) {
-                        return i;
-                    }
-                }
-                
-                return -1;
-            }
-        }
-        
         public static ICollection Sources {
-            get {
-                return sources;
-            }
+            get { return sources; }
         }
     }
 }
