@@ -1,9 +1,8 @@
-
 /***************************************************************************
- *  LibrarySource.cs
+ *  LocalQueueSource.cs
  *
- *  Copyright (C) 2005 Novell
- *  Written by Aaron Bockover (aaron@aaronbock.net)
+ *  Copyright (C) 2005-2006 Novell, Inc.
+ *  Written by Aaron Bockover <aaron@abock.org>
  ****************************************************************************/
 
 /*  THIS FILE IS LICENSED UNDER THE MIT LICENSE AS OUTLINED IMMEDIATELY BELOW: 
@@ -29,6 +28,7 @@
  
 using System;
 using System.Collections;
+using System.Collections.Generic;
 using Mono.Unix;
 
 using Banshee.Base;
@@ -37,7 +37,7 @@ namespace Banshee.Sources
 {
     public class LocalQueueSource : Source
     {
-        private ArrayList tracks = new ArrayList();
+        private List<TrackInfo> tracks = new List<TrackInfo>();
         
         private static LocalQueueSource instance;
         public static LocalQueueSource Instance {
@@ -61,19 +61,25 @@ namespace Banshee.Sources
         
             foreach(string file in files) {
                 try {
-                    Uri uri = file.StartsWith("file://") ? new Uri(file) : PathUtil.PathToFileUri(file);
-                    if(!System.IO.File.Exists(uri.LocalPath)) {
-                        uri = PathUtil.PathToFileUri(System.Environment.CurrentDirectory + 
+                    SafeUri uri;
+                    try {
+                        uri = new SafeUri(file);
+                    } catch(ApplicationException) {
+                        uri = new SafeUri(System.Environment.CurrentDirectory + 
                             System.IO.Path.DirectorySeparatorChar + file);
                     }
                     
-                    TrackInfo track = new FileTrackInfo(uri);
-                    tracks.Add(track);
-                    OnTrackAdded(track);
-                    
-                    if(playFirst && !played_first) {
-                        played_first = true;
-                        PlayerEngineCore.OpenPlay(track);
+                    if(System.IO.File.Exists(uri.LocalPath)) {
+                        TrackInfo track = new FileTrackInfo(uri);
+                        tracks.Add(track);
+                        OnTrackAdded(track);
+                        
+                        if(playFirst && !played_first) {
+                            played_first = true;
+                            PlayerEngineCore.OpenPlay(track);
+                        }  
+                    } else {
+                        throw new ApplicationException(uri.LocalPath);
                     }
                 } catch(Exception e) {
                     Console.WriteLine("Could not load: {0}", e.Message);
@@ -87,27 +93,18 @@ namespace Banshee.Sources
             OnUpdated();
         }
         
-        public override void RemoveTrack(TrackInfo track)
-        {
-            tracks.Remove(track);
-        }
-        
         public override int Count {
-            get {
-                return tracks.Count;
-            }
+            get { return tracks.Count; }
         }
         
-        public override IEnumerable Tracks {
-            get {
-                return tracks;
-            }
+        public override IEnumerable<TrackInfo> Tracks {
+            get { return tracks; }
         }
 
+        private static Gdk.Pixbuf icon = IconThemeUtils.LoadIcon(22, "system-file-manager", "source-localqueue");
         public override Gdk.Pixbuf Icon {
-            get {
-                return IconThemeUtils.LoadIcon(22, "system-file-manager", "source-localqueue");
-            }
+            get { return icon; }
         }
     }
 }
+
