@@ -29,36 +29,50 @@
 using System;
 using System.Data;
 using System.Collections;
+using System.Collections.Generic;
 using Mono.Unix;
 
 using Banshee.Base;
 
 namespace Banshee.Sources
 {
-    public class PlaylistSource : Source
+    public class PlaylistSource : ChildSource
     {
-        private static ArrayList playlists = new ArrayList();
-        public static IEnumerable Playlists {
-            get {
-                return playlists;
-            }
+        private static List<PlaylistSource> playlists = new List<PlaylistSource>();
+    
+        public static IEnumerable<PlaylistSource> Playlists {
+            get { return playlists; }
         }
         
         public static int PlaylistCount {
-            get {
-                return playlists.Count;
-            }
+            get { return playlists.Count; }
         }
         
-        private ArrayList tracks = new ArrayList();
-        private Queue remove_queue = new Queue();
-        private Queue append_queue = new Queue();
+        private List<TrackInfo> tracks = new List<TrackInfo>();
+        private Queue<TrackInfo> remove_queue = new Queue<TrackInfo>();
+        private Queue<TrackInfo> append_queue = new Queue<TrackInfo>();
         private int id;
-    
+
+        public int Id {
+            get { return id; }
+        }
+
+        public override string UnmapLabel {
+            get { return Catalog.GetString("Delete Playlist"); }
+        }
+
+        public override string GenericName {
+            get { return Catalog.GetString("Playlist"); }
+        }
+
         public PlaylistSource() : this(0)
         {
         }
-    
+
+        public PlaylistSource(string name) : base(name, 500)
+        {
+        }
+
         public PlaylistSource(int id) : base(Catalog.GetString("New Playlist"), 500)
         {
             this.id = id;
@@ -68,12 +82,21 @@ namespace Banshee.Sources
             } else if(id == 0) {
                 CreateNewPlaylist();
             } else {
-                LoadFromDatabase();
+                if(Globals.Library.IsLoaded) {
+                    LoadFromDatabase();
+                } else {
+                    Globals.Library.Reloaded += OnLibraryReloaded;
+                }
             }
             
             Globals.Library.TrackRemoved += OnLibraryTrackRemoved;
             playlists.Add(this);
         }
+
+        private void OnLibraryReloaded (object o, EventArgs args)
+        {
+            LoadFromDatabase();
+        }
         
         private void CreateNewPlaylist()
         {
@@ -170,6 +193,11 @@ namespace Banshee.Sources
                 OnUpdated();
             }
         }
+
+        public void ClearTracks()
+        {
+            tracks.Clear();
+        }
         
         public override void RemoveTrack(TrackInfo track)
         {
@@ -179,7 +207,23 @@ namespace Banshee.Sources
             }
         }
         
-        public void Delete()
+        public override void SourceDrop(Source source)
+        {
+            if(source == this || !(source is PlaylistSource)) {
+                return;
+            }
+            
+            foreach(TrackInfo track in source.Tracks) {
+                AddTrack(track);
+            }
+        }
+
+        public bool ContainsTrack(TrackInfo track)
+        {
+            return tracks.Contains(track);
+        }
+        
+        public override bool Unmap()
         {
             Globals.Library.Db.Execute(String.Format(
                 @"DELETE FROM PlaylistEntries
@@ -199,6 +243,8 @@ namespace Banshee.Sources
             
             SourceManager.RemoveSource(this);
             playlists.Remove(this);
+
+            return true;
         }
         
         public override void Commit()
@@ -206,13 +252,14 @@ namespace Banshee.Sources
             if(remove_queue.Count > 0) {
                 lock(TracksMutex) {
                     while(remove_queue.Count > 0) {
-                        TrackInfo track = remove_queue.Dequeue() as TrackInfo;
+                        TrackInfo track = remove_queue.Dequeue();
                         Globals.Library.Db.Execute(String.Format(
                             @"DELETE FROM PlaylistEntries
                                 WHERE PlaylistID = '{0}'
                                 AND TrackID = '{1}'",
                                 id, track.TrackId
                         ));
+                        OnTrackRemoved(track);
                     }
                 }
             }
@@ -220,7 +267,7 @@ namespace Banshee.Sources
             if(append_queue.Count > 0) {
                 lock(TracksMutex) {
                     while(append_queue.Count > 0) {
-                        TrackInfo track = append_queue.Dequeue() as TrackInfo;
+                        TrackInfo track = append_queue.Dequeue();
                         Globals.Library.Db.Execute(String.Format(
                             @"INSERT INTO PlaylistEntries 
                                 VALUES (NULL, '{0}', '{1}', (
@@ -231,6 +278,7 @@ namespace Banshee.Sources
                                     WHERE PlaylistID = '{0}')
                                 )", id, track.TrackId
                         ));
+                        OnTrackAdded(track);
                     }
                 }
             }
@@ -242,7 +290,7 @@ namespace Banshee.Sources
                 int sql_position = 1;
             
                 if(position > 0) {
-                    TrackInfo sibling = tracks[position] as TrackInfo;
+                    TrackInfo sibling = tracks[position];
                     if(sibling == track || sibling == null) {
                         return;
                     }
@@ -286,7 +334,7 @@ namespace Banshee.Sources
                     RemoveTrack(args.Track);
                     
                     if(Count == 0) {
-                        Delete();
+                        Unmap();
                     } else {
                         Commit();
                     }
@@ -311,46 +359,51 @@ namespace Banshee.Sources
             
             if(removed_count > 0) {
                 if(Count == 0) {
-                    Delete();
+                    Unmap();
                 } else {
                     Commit();
                 }
             }
         }
         
-        public override IEnumerable Tracks {
-            get {
-                return tracks;
-            }
+        public override IEnumerable<TrackInfo> Tracks {
+            get { return tracks; }
         }
         
         public override object TracksMutex {
-            get { return tracks.SyncRoot; }
+            get { return ((IList)tracks).SyncRoot; }
         }
         
         public override int Count {
-            get {
-                return tracks.Count;
-            }
+            get { return tracks.Count; }
         }  
         
+        public override bool IsDragSource {
+            get { return true; }
+        }
+        
+        private static Gdk.Pixbuf icon = IconThemeUtils.LoadIcon(22, "source-playlist");
         public override Gdk.Pixbuf Icon {
-            get {
-                return IconThemeUtils.LoadIcon(22, "source-playlist");
-            }
+            get { return icon; }
         }
     }
     
     public static class PlaylistUtil
     {
-        public static void LoadSources()
+        public static ICollection LoadSources()
         {
-            IDataReader reader = Globals.Library.Db.Query("SELECT PlaylistID FROM Playlists");
+            ArrayList sources = new ArrayList();
+            IDataReader reader = Globals.Library.Db.Query(@"
+                SELECT PlaylistID FROM Playlists"
+            );
+            
             while(reader.Read()) {
                 PlaylistSource playlist = new PlaylistSource(Convert.ToInt32(reader[0]));
-                SourceManager.AddSource(playlist);
+                sources.Add(playlist);
             }
+            
             reader.Dispose();
+            return sources;
         }
     
         internal static int GetPlaylistID(string name)
@@ -377,92 +430,14 @@ namespace Banshee.Sources
             return GetPlaylistID(name) > 0;
         }
         
-        internal static string PostfixDuplicate(string prefix)
-        {
-            string name = prefix;
-            for(int i = 1; true; i++) {
-                if(!PlaylistExists(name)) {
-                    return name;
-                }
-                
-                name = prefix + " " + i;
-            }
-        }
-        
         public static string UniqueName {
-            get {
-                return PostfixDuplicate(Catalog.GetString("New Playlist"));
-            }
+            get { return NamingUtil.PostfixDuplicate(Catalog.GetString("New Playlist"), PlaylistExists); }
         }
         
         public static string GoodUniqueName(IEnumerable tracks)
         {
-            ArrayList names = new ArrayList();
-            Hashtable groups = new Hashtable();
-            
-            int track_count = 0;
-            
-            foreach(TrackInfo ti in tracks) {
-                bool haveArtist = ti.Artist != null && !ti.Artist.Equals(String.Empty);
-                bool haveAlbum = ti.Album != null && !ti.Album.Equals(String.Empty);
-            
-                if(haveArtist && haveAlbum) {
-                    names.Add(ti.Artist + " - " + ti.Album);
-                } else if(haveArtist) {
-                    names.Add(ti.Artist);
-                } else if(haveAlbum) {
-                    names.Add(ti.Album);
-                } else {
-                    names.Add(Catalog.GetString("New Playlist"));
-                }
-                
-                track_count++;
-            }
-            
-            if(track_count == 0) {
-                names.Add(Catalog.GetString("New Playlist"));
-            }
-                
-            names.Sort();
-            groups[names[0]] = 1;
-            
-            for(int i = 1; i < names.Count; i++) {
-                bool match = false;
-                foreach(string key in groups.Keys) {
-                    if(names[i].Equals(key)) {
-                        groups[key] = ((int)groups[key]) + 1;
-                        match = true;
-                        break;
-                    }
-                }
-            
-                if(match) {
-                    continue;
-                }
-                
-                groups[names[i]] = 1;
-            }
-            
-            string bestMatch = String.Empty;
-            int maxValue = 0;
-            
-            foreach(int count in groups.Values) {
-                if(count > maxValue) {
-                    maxValue = count;
-                    foreach(string key in groups.Keys) {
-                        if((int)groups[key] == maxValue) {
-                            bestMatch = key;
-                            break;
-                        }
-                    }
-                }
-            }
-            
-            if(bestMatch.Equals(String.Empty)) {
-                return UniqueName;
-            }
-                
-            return PostfixDuplicate(bestMatch);
+            return NamingUtil.PostfixDuplicate(NamingUtil.GenerateTrackCollectionName(
+                tracks, Catalog.GetString("New Playlist")), PlaylistExists);
         }
     }
 }
