@@ -1,9 +1,8 @@
-
 /***************************************************************************
  *  IpodDap.cs
  *
- *  Copyright (C) 2005 Novell
- *  Written by Aaron Bockover (aaron@aaronbock.net)
+ *  Copyright (C) 2005-2006 Novell, Inc.
+ *  Written by Aaron Bockover <aaron@abock.org>
  ****************************************************************************/
 
 /*  THIS FILE IS LICENSED UNDER THE MIT LICENSE AS OUTLINED IMMEDIATELY BELOW: 
@@ -30,6 +29,8 @@
 using System; 
 using System.IO;
 using System.Collections;
+using System.Reflection;
+using System.Diagnostics;
 using Mono.Unix;
 using Gtk;
 using Hal;
@@ -37,6 +38,7 @@ using IPod;
 
 using Banshee.Base;
 using Banshee.Dap;
+using Banshee.Widgets;
  
 namespace Banshee.Dap.Ipod
 {
@@ -73,8 +75,19 @@ namespace Banshee.Dap.Ipod
             InstallProperty("Model Number", device.ModelNumber);
             InstallProperty("Serial Number", device.SerialNumber);
             InstallProperty("Firmware Version", device.FirmwareVersion);
-            InstallProperty("Database Version", device.SongDatabase.Version.ToString());
-          
+            InstallProperty("Database Version", device.TrackDatabase.Version.ToString());
+            
+            if(device.ProductionYear > 0) {
+                // Translators "Week 25 of 2006"
+                InstallProperty(Catalog.GetString("Manufactured During"), 
+                    String.Format(Catalog.GetString("Week {0} of {1}"), device.ProductionWeek, 
+                        device.ProductionYear.ToString("0000")));
+            }
+            
+            if(device.ShouldAskIfUnknown) {
+                GLib.Timeout.Add(5000, AskAboutUnknown);
+            }
+
             ReloadDatabase(false);
             
             CanCancelSave = false;
@@ -85,31 +98,69 @@ namespace Banshee.Dap.Ipod
         {
             try {
                 device = new IPod.Device(hal_device["block.device"]);
-                device.LoadSongDatabase();
+                device.LoadTrackDatabase();
                 database_supported = true;
             } catch(DatabaseReadException) {
-                device.LoadSongDatabase(true);
+                device.LoadTrackDatabase(true);
                 database_supported = false;
-            } catch(Exception e) {
+            } catch {
                 return InitializeResult.Invalid;
             }
             
             return InitializeResult.Valid;
         }
         
-        protected override TrackInfo OnTrackAdded(TrackInfo track)
+        private bool AskAboutUnknown()
         {
-            if(track is IpodDapTrackInfo) {
-                return track;
-            }
-
-            if(!TrackExistsInList(track, device.SongDatabase.Songs)) {
-                return new IpodDapTrackInfo(track, device.SongDatabase);
+            HigMessageDialog dialog = new HigMessageDialog(null, Gtk.DialogFlags.Modal,
+                Gtk.MessageType.Warning, Gtk.ButtonsType.None,
+                Catalog.GetString("Your iPod could not be identified"),
+                Catalog.GetString("Please consider submitting information about your iPod " +
+                    "to the Banshee Project so your iPod may be more fully identified in the future.\n"));
+        
+            CheckButton do_not_ask = new CheckButton(Catalog.GetString("Do not ask me again"));
+            do_not_ask.Show();
+            dialog.LabelVBox.PackStart(do_not_ask, false, false, 0);
+            
+            dialog.AddButton("gtk-cancel", Gtk.ResponseType.Cancel, false);
+            dialog.AddButton(Catalog.GetString("Go to Web Site"), Gtk.ResponseType.Ok, false);
+            
+            try {
+                if(dialog.Run() == (int)ResponseType.Ok) {
+                    do_not_ask.Active = true;
+                    Gnome.Url.Show(device.UnknownIpodUrl);
+                }
+            } finally {
+                dialog.Destroy();
             }
             
-            return null;
+            if(do_not_ask.Active) {
+                device.DoNotAskIfUnknown();
+            }
+            
+            return false;
         }
         
+        public override void AddTrack(TrackInfo track)
+        {
+            if (track == null || IsReadOnly)
+                return;
+
+            TrackInfo new_track = null;
+
+            if(track is IpodDapTrackInfo)
+                new_track = track;
+            else
+                new_track = new IpodDapTrackInfo(track, device.TrackDatabase);
+            // FIXME: only add a new track if we don't have it already
+
+
+            if (new_track != null) {
+                tracks.Add(new_track);
+                OnTrackAdded(new_track);
+            }
+        }
+
         protected override void OnTrackRemoved(TrackInfo track)
         {
             if(!(track is IpodDapTrackInfo)) {
@@ -118,7 +169,7 @@ namespace Banshee.Dap.Ipod
             
             try {
                 IpodDapTrackInfo ipod_track = (IpodDapTrackInfo)track;
-                device.SongDatabase.RemoveSong(ipod_track.Song);
+                device.TrackDatabase.RemoveTrack(ipod_track.Track);
             } catch(Exception) {
             }
         }
@@ -130,13 +181,13 @@ namespace Banshee.Dap.Ipod
             ClearTracks(false);
             
             if(refresh) {
-                device.SongDatabase.Reload();
+                device.TrackDatabase.Reload();
             }
             
             if(database_supported) {
-                foreach(Song song in device.SongDatabase.Songs) {
-                    IpodDapTrackInfo track = new IpodDapTrackInfo(song);
-                    AddTrack(track);            
+                foreach(Track track in device.TrackDatabase.Tracks) {
+                    IpodDapTrackInfo ti = new IpodDapTrackInfo(track);
+                    AddTrack(ti);            
                 }
             } else {
                 BuildDatabaseUnsupportedWidget();
@@ -149,8 +200,13 @@ namespace Banshee.Dap.Ipod
         
         public override void Eject()
         {
-            device.Eject();
-            base.Eject();
+            try {
+                device.Eject();
+                base.Eject();
+            } catch(Exception e) {
+                LogCore.Instance.PushError(Catalog.GetString("Could not eject iPod"),
+                    e.Message);
+            }
         }
         
         public override void Synchronize()
@@ -161,25 +217,25 @@ namespace Banshee.Dap.Ipod
                 0.0);
             
             foreach(IpodDapTrackInfo track in Tracks) {
-                if(track.Song == null) {
+                if(track.Track == null) {
                     CommitTrackToDevice(track);
                 } else {
-                    track.Song.Uri = track.Uri;
+                    track.Track.Uri = new Uri(track.Uri.AbsoluteUri);
                 }
             }
             
-            device.SongDatabase.SaveProgressChanged += delegate(object o, SaveProgressArgs args)
+            device.TrackDatabase.SaveProgressChanged += delegate(object o, TrackSaveProgressArgs args)
             {
-                double progress = args.CurrentSong == null ? 0.0 : args.TotalProgress;
-                string message = args.CurrentSong == null 
+                double progress = args.CurrentTrack == null ? 0.0 : args.TotalProgress;
+                string message = args.CurrentTrack == null 
                     ? Catalog.GetString("Flushing to Disk (may take time)")
-                    : args.CurrentSong.Artist + " - " + args.CurrentSong.Title;
+                    : args.CurrentTrack.Artist + " - " + args.CurrentTrack.Title;
                     
                 UpdateSaveProgress(Catalog.GetString("Synchronizing iPod"), message, progress);
             };
 
             try {
-                device.SongDatabase.Save();
+                device.TrackDatabase.Save();
             } catch(Exception e) {
                 Console.Error.WriteLine (e);
                 LogCore.Instance.PushError(Catalog.GetString("Failed to synchronize iPod"), e.Message);
@@ -189,57 +245,84 @@ namespace Banshee.Dap.Ipod
             }
         }
         
-        private void CommitTrackToDevice(IpodDapTrackInfo track)
+        private void CommitTrackToDevice(IpodDapTrackInfo ti)
         {
-            Song song = device.SongDatabase.CreateSong();
-            
-            song.Uri = track.Uri;
+            Track track = device.TrackDatabase.CreateTrack();
+
+            try {
+                track.Uri = new Uri(ti.Uri.AbsoluteUri);
+            } catch {
+                device.TrackDatabase.RemoveTrack (track);
+                return;
+            }
         
-            if(track.Album != null) {
-                song.Album = track.Album;
+            if(ti.Album != null) {
+                track.Album = ti.Album;
             }
             
-            if(track.Artist != null) {
-                song.Artist = track.Artist;
+            if(ti.Artist != null) {
+                track.Artist = ti.Artist;
             }
             
-            if(track.Title != null) {
-                song.Title = track.Title;
+            if(ti.Title != null) {
+                track.Title = ti.Title;
             }
             
-            if(track.Genre != null) {
-                song.Genre = track.Genre;
+            if(ti.Genre != null) {
+                track.Genre = ti.Genre;
             }
             
-            song.Duration = track.Duration;
-            song.TrackNumber = (int)track.TrackNumber;
-            song.TotalTracks = (int)track.TrackCount;
-            song.Year = (int)track.Year;
-            song.LastPlayed = track.LastPlayed;
+            track.Duration = ti.Duration;
+            track.TrackNumber = (int)ti.TrackNumber;
+            track.TotalTracks = (int)ti.TrackCount;
+            track.Year = (int)ti.Year;
+            track.LastPlayed = ti.LastPlayed;
             
-            switch(track.Rating) {
-                case 1: song.Rating = SongRating.Zero; break;
-                case 2: song.Rating = SongRating.Two; break;
-                case 3: song.Rating = SongRating.Three; break;
-                case 4: song.Rating = SongRating.Four; break;
-                case 5: song.Rating = SongRating.Five; break;
-                default: song.Rating = SongRating.Zero; break;
+            switch(ti.Rating) {
+                case 1: track.Rating = TrackRating.Zero; break;
+                case 2: track.Rating = TrackRating.Two; break;
+                case 3: track.Rating = TrackRating.Three; break;
+                case 4: track.Rating = TrackRating.Four; break;
+                case 5: track.Rating = TrackRating.Five; break;
+                default: track.Rating = TrackRating.Zero; break;
             }
             
-            if(song.Artist == null) {
-                song.Artist = String.Empty;
+            if(track.Artist == null) {
+                track.Artist = String.Empty;
             }
             
-            if(song.Album == null) {
-                song.Album = String.Empty;
+            if(track.Album == null) {
+                track.Album = String.Empty;
             }
             
-            if(song.Title == null) {
-                song.Title = String.Empty;
+            if(track.Title == null) {
+                track.Title = String.Empty;
             }
             
-            if(song.Genre == null) {
-                song.Genre = String.Empty;
+            if(track.Genre == null) {
+                track.Genre = String.Empty;
+            }
+
+            if (ti.CoverArtFileName != null && File.Exists (ti.CoverArtFileName)) {
+                try {
+                    Gdk.Pixbuf pixbuf = new Gdk.Pixbuf (ti.CoverArtFileName);
+
+                    if (pixbuf != null) {
+                        SetCoverArt (track, ArtworkUsage.Cover, pixbuf);
+                        pixbuf.Dispose ();
+                    }
+                } catch (Exception e) {
+                    Console.Error.WriteLine ("Failed to set cover art from {0}: {1}", ti.CoverArtFileName, e);
+                }
+            }
+        }
+
+        private void SetCoverArt (Track track, ArtworkUsage usage, Gdk.Pixbuf pixbuf)
+        {
+            foreach (ArtworkFormat format in device.LookupArtworkFormats (usage)) {
+                if (!track.HasCoverArt (format)) {
+                    track.SetCoverArt (format, ArtworkHelpers.ToBytes (format, pixbuf));
+                }
             }
         }
         
