@@ -44,8 +44,12 @@ namespace Banshee.Dap.MassStorage
     {
         private static Gnome.Vfs.VolumeMonitor monitor;
 
+        private bool mounted = false, ui_initialized = false;
+
         static MassStorageDap () {
-            Gnome.Vfs.Vfs.Initialize ();
+            if (!Gnome.Vfs.Vfs.Initialized)
+                Gnome.Vfs.Vfs.Initialize();
+
             monitor = Gnome.Vfs.VolumeMonitor.Get ();
         }
 
@@ -63,52 +67,81 @@ namespace Banshee.Dap.MassStorage
                 player_device = Hal.Device.UdisToDevices (volume_device.Context, new string [] {volume_device ["info.parent"]}) [0];
                 usb_device = Hal.Device.UdisToDevices (player_device.Context, new string [] {player_device ["storage.physical_device"]}) [0];
             } catch (Exception e) {
-                    return InitializeResult.Invalid;
-            }
-
-            if (!player_device.PropertyExists ("portable_audio_player.access_method") ||
-                player_device ["portable_audio_player.access_method"] != "storage" ||
-                !usb_device.PropertyExists("usb.vendor_id") ||
-                !usb_device.PropertyExists("usb.product_id") ||
-                !volume_device.PropertyExists("block.device")) {
                 return InitializeResult.Invalid;
             }
 
-            if(!volume_device.PropertyExists ("volume.is_mounted") ||
-                    !volume_device.GetPropertyBool("volume.is_mounted"))
-                return InitializeResult.WaitForPropertyChange;
-	    
+            if (!volume_device.PropertyExists("block.device"))
+                return InitializeResult.Invalid;
+            
+            if (!volume_device.PropertyExists ("volume.is_mounted") ||
+                !volume_device.GetPropertyBool("volume.is_mounted"))
+                 return InitializeResult.WaitForPropertyChange;
 
-            string block_device = volume_device ["block_device"];
-            foreach (Gnome.Vfs.Volume vol in monitor.MountedVolumes) {
-                if (vol.DevicePath == block_device) {
-                    this.volume = vol;
-                    break;
-                }
+            // Detect player via HAL property or presence of .is_audo_player file in root
+            if (player_device["portable_audio_player.access_method"] != "storage" &&
+                !File.Exists(Path.Combine(MountPoint, ".is_audio_player"))) {                
+                 return InitializeResult.Invalid;
             }
 
-            if (volume == null)
-                return InitializeResult.Invalid;
-
-            is_read_only = volume.IsReadOnly;
+            volume = monitor.GetVolumeForPath(MountPoint);
+            if (volume == null) {
+                // Gnome VFS doesn't know volume is mounted yet
+                monitor.VolumeMounted += OnVolumeMounted;
+                is_read_only = true;
+            } else {
+                mounted = true;
+                is_read_only = volume.IsReadOnly;
+            }
 
             base.Initialize (usb_device);
  
-            InstallProperty("Vendor", usb_device["usb.vendor"]);
+            if (usb_device.PropertyExists("usb.vendor"))
+                InstallProperty(Catalog.GetString("Vendor"), usb_device["usb.vendor"]);
+            else if (player_device.PropertyExists("info.vendor"))
+                InstallProperty(Catalog.GetString("Vendor"), player_device["info.vendor"]);
+
+            if(!Globals.UIManager.IsInitialized) {
+                Globals.UIManager.Initialized += OnUIManagerInitialized;
+            } else {
+                ui_initialized = true;
+            }
+
+            if (ui_initialized && mounted)
+                ReloadDatabase ();
 
-            ReloadDatabase();
-            
             // FIXME probably should be able to cancel at some point when you can actually sync
             CanCancelSave = false;
             return InitializeResult.Valid;
         }
 
+        public void OnVolumeMounted(object o, Gnome.Vfs.VolumeMountedArgs args)
+        {
+            if(args.Volume.DevicePath == volume_device["block.device"]) {
+                monitor.VolumeMounted -= OnVolumeMounted;
+            
+                volume = args.Volume;
+                is_read_only = volume.IsReadOnly;
+
+                mounted = true;
+
+                if (ui_initialized)
+                    ReloadDatabase ();
+            }
+        }
+
         public override void Dispose()
         {
             // FIXME anything else to do here?
             volume = null;
             base.Dispose();
         }
+
+        private void OnUIManagerInitialized(object o, EventArgs args)
+        {
+            ui_initialized = true;
+            if (mounted)
+                ReloadDatabase ();
+        }
  
         private void ReloadDatabase()
         {
@@ -124,13 +157,13 @@ namespace Banshee.Dap.MassStorage
         private void HandleImportRequested (object o, ImportEventArgs args)
         {
             try {
-                TrackInfo track = new MassStorageTrackInfo (new Uri (args.FileName));
+                TrackInfo track = new MassStorageTrackInfo (new SafeUri (args.FileName));
                 args.ReturnMessage = String.Format("{0} - {1}", track.Artist, track.Title);
 
                 AddTrack (track);
-            } catch(Entagged.Audioformats.Exceptions.CannotReadException) {
-                //Console.WriteLine(Catalog.GetString("Cannot Import") + ": {0}", args.FileName);
-                args.ReturnMessage = Catalog.GetString("Scanning") + "...";
+            /* TODO: TagLib exception? } catch(Entagged.Audioformats.Exceptions.CannotReadException) {
+                //Console.WriteLine(Catalog.GetString("Cannot Import") + ": {0}", args.FileName); 
+                args.ReturnMessage = Catalog.GetString("Scanning") + "..."; */
             } catch(Exception e) {
                 //Console.WriteLine(Catalog.GetString("Cannot Import: {0} ({1}, {2})"), 
                     //args.FileName, e.GetType(), e.Message);
@@ -145,7 +178,8 @@ namespace Banshee.Dap.MassStorage
 
         public override void Eject ()
         {
-            volume.Unmount (UnmountCallback);
+            if(volume != null)
+                volume.Unmount (UnmountCallback);
         }
 
         private void UnmountCallback (bool succeeded, string error, string detailed_error)
@@ -153,7 +187,7 @@ namespace Banshee.Dap.MassStorage
             if (succeeded)
                 volume.Eject (EjectCallback);
             else
-                Console.WriteLine ("Failed to unmount.  {1} {2}", error, detailed_error);
+                Console.WriteLine ("Failed to unmount.  {0} {1}", error, detailed_error);
         }
 
         private void EjectCallback (bool succeeded, string error, string detailed_error)
@@ -161,21 +195,54 @@ namespace Banshee.Dap.MassStorage
             if (succeeded)
                 base.Eject ();
             else
-                Console.WriteLine ("Failed to eject.  {1} {2}", error, detailed_error);
+                Console.WriteLine ("Failed to eject.  {0} {1}", error, detailed_error);
         }
-        
-        protected override TrackInfo OnTrackAdded(TrackInfo track)
+
+        public override void AddTrack(TrackInfo track)
         {
-            if (track is MassStorageTrackInfo || IsReadOnly)
-                return track;
+            if (track == null || IsReadOnly)
+                return;
 
-            string new_path = GetTrackPath (track);
-            Directory.CreateDirectory (Path.GetDirectoryName (new_path));
-            File.Copy (track.Uri.LocalPath, new_path);
-
-            return new MassStorageTrackInfo (new Uri (new_path));
+            if (track is MassStorageTrackInfo) {
+                // If we're "adding" it when it's already on the device, then
+                // we don't need to copy it
+                tracks.Add(track);
+                OnTrackAdded(track);
+            } else {
+                // Otherwise queue it up to be transferred over to the device
+                Copier.Enqueue (track);
+            }
         }
         
+        private void HandleCopyRequested (object o, QueuedOperationArgs args)
+        {
+            TrackInfo track = args.Object as TrackInfo;
+
+            if (track == null)
+                return;
+            
+            try {
+                string new_path = GetTrackPath (track);
+
+                // If it already is on the device but it's out of date, remove it
+                if (File.Exists (new_path) && File.GetLastWriteTime(track.Uri.LocalPath) > File.GetLastWriteTime(new_path))
+                    RemoveTrack(new MassStorageTrackInfo(new SafeUri(new_path)));
+
+                if (!File.Exists (new_path)) {
+                        Directory.CreateDirectory (Path.GetDirectoryName (new_path));
+                        File.Copy (track.Uri.LocalPath, new_path);
+                }
+
+                TrackInfo new_track = new MassStorageTrackInfo (new SafeUri (new_path));
+                tracks.Add(new_track);
+                OnTrackAdded(new_track);
+
+                args.ReturnMessage = String.Format("{0} - {1}", track.Artist, track.Title);
+            } catch (Exception e) {
+                args.ReturnMessage = String.Format("Skipping Song", track.Artist, track.Title);
+            }
+        }
+
         protected override void OnTrackRemoved(TrackInfo track)
         {
             if (IsReadOnly)
@@ -212,26 +279,71 @@ namespace Banshee.Dap.MassStorage
 
         private string GetTrackPath (TrackInfo track)
         {
-            string file_path = "";
-            if (player_device.PropertyExists ("portable_audio_player.filepath_format")) {
-                file_path = player_device.GetPropertyString ("portable_audio_player.filepath_format");
-                file_path = file_path.Replace ("%Artist", track.Artist);
-                file_path = file_path.Replace ("%Album", track.Album);
+            string file_path = MountPoint;
 
-                if (file_path.IndexOf ("%Track") == -1) {
-                    file_path = System.IO.Path.Combine (file_path, track.TrackNumberTitle);
+            // According to the HAL spec, the first folder listed in the audio_folders property
+            // is the folder to write files to.
+            if (AudioFolders != null && AudioFolders.Length > 0) {
+                file_path = System.IO.Path.Combine (file_path, AudioFolders[0]);
+            }
+
+            string artist = FileNamePattern.Escape (track.Artist);
+            string album = FileNamePattern.Escape (track.Album);
+            string number_title = FileNamePattern.Escape (track.TrackNumberTitle);
+
+            // If the folder_depth property exists, we have to put the files in a hiearchy of
+            // the exact given depth (not including the mount point/audio_folder).
+            if (player_device.PropertyExists ("portable_audio_player.folder_depth")) {
+                int depth = player_device.GetPropertyInt ("portable_audio_player.folder_depth");
+
+                if (depth == 0) {
+                    // Artist - Album - 01 - Title
+                    file_path = System.IO.Path.Combine (file_path, String.Format ("{0} - {1} - {2}", artist, album, number_title));
+                } else if (depth == 1) {
+                    // Artist - Album/01 - Title
+                    file_path = System.IO.Path.Combine (file_path, String.Format ("{0} - {1}", artist, album));
+                    file_path = System.IO.Path.Combine (file_path, number_title);
+                } else if (depth == 2) {
+                    // Artist/Album/01 - Title
+                    file_path = System.IO.Path.Combine (file_path, artist);
+                    file_path = System.IO.Path.Combine (file_path, album);
+                    file_path = System.IO.Path.Combine (file_path, number_title);
                 } else {
-                    file_path = file_path.Replace ("%Track", track.TrackNumberTitle);
+                    // If the *required* depth is more than 2..go nuts!
+                    for (int i = 0; i < depth - 2; i++) {
+                        file_path = System.IO.Path.Combine (file_path, artist.Substring (0, Math.Min (i, artist.Length)));
+                    }
+
+                    // Finally add on the Artist/Album/01 - Track
+                    file_path = System.IO.Path.Combine (file_path, artist);
+                    file_path = System.IO.Path.Combine (file_path, album);
+                    file_path = System.IO.Path.Combine (file_path, number_title);
                 }
             } else {
-                file_path = System.IO.Path.Combine (track.Artist, track.Album);
-                file_path = System.IO.Path.Combine (file_path, track.TrackNumberTitle);
+                file_path = System.IO.Path.Combine (file_path, artist);
+                file_path = System.IO.Path.Combine (file_path, album);
+                file_path = System.IO.Path.Combine (file_path, number_title);
             }
+                    
 
             file_path += Path.GetExtension (track.Uri.LocalPath);
 
-            //Console.WriteLine ("for track {0} outpath is {1}", track.Uri.LocalPath, System.IO.Path.Combine (MountPoint, file_path));
-            return System.IO.Path.Combine (MountPoint, file_path);
+            return file_path;
+        }
+
+        private QueuedOperationManager copier;
+        public QueuedOperationManager Copier {
+            get {
+                if (copier == null) {
+                    copier = new QueuedOperationManager ();
+                    copier.ActionMessage = Catalog.GetString ("Copying Songs");
+                    copier.ProgressMessage = Catalog.GetString ("Copying {0} of {1}");
+                    copier.OperationRequested += HandleCopyRequested;
+                }
+
+                return copier;
+            }
+            set { copier = value; }
         }
 
         public virtual string IconId {
@@ -242,12 +354,14 @@ namespace Banshee.Dap.MassStorage
  
         public override string Name {
             get {
-                if (volume_device.PropertyExists("volume.label"))
-                    return volume_device["volume.label"];
-
                 if (player_device.PropertyExists("info.product"))
                     return player_device["info.product"];
 
+                if (volume_device.PropertyExists("volume.label") &&
+                    volume_device["volume.label"].Length > 0)
+                    return volume_device["volume.label"];
+
+
                 return GenericName;
             }
         }
