@@ -1,13 +1,11 @@
 /***************************************************************************
  *  NotificationAreaIconPlugin.cs
  *
- *  Copyright (C) 2005-2006 Novell, Inc. (Aaron Bockover)
- *  Copyright (C) 2006 Sebastian Dröge <slomo@ubuntu.com>
+ *  Copyright (C) 2005-2006 Novell, Inc.
+ *  Copyright (C) 2006 Sebastian Dröge
  * 
  *  Written by Sebastian Dröge <slomo@ubuntu.com>
- *  
- *  Parts of the file copied from the old TrayIcon code,
- *  written by Aaron Bockover <aaron@aaronbock.net>
+ *             Aaron Bockover <aaron@abock.org>
  *  
  ****************************************************************************/
 
@@ -41,24 +39,23 @@ using Banshee.MediaEngine;
 using Banshee.Widgets;
 using Mono.Unix;
 
-namespace Banshee.Plugins.NotificationAreaIcon {
-
-    public class NotificationAreaIconPlugin : Banshee.Plugins.Plugin {
-
+namespace Banshee.Plugins.NotificationAreaIcon 
+{
+    public class NotificationAreaIconPlugin : Banshee.Plugins.Plugin 
+    {
         protected override string ConfigurationName { get { return "NotificationAreaIcon"; } }
         public override string DisplayName { get { return Catalog.GetString("Notification Area Icon"); } }
 
         public override string Description {
-            get {
-                return Catalog.GetString("Shows the Notification Area Icon");
-            }
+            get { return Catalog.GetString("Shows the Notification Area Icon"); }
         }
 
         public override string[] Authors {
             get {
                 return new string[] {
                     "Sebastian Dr\u00f6ge",
-                    "Aaron Bockover"
+                    "Aaron Bockover",
+                    "Ruben Vermeersch"
                 };
             }
         }
@@ -71,11 +68,14 @@ namespace Banshee.Plugins.NotificationAreaIcon {
         private TrackInfoPopup popup;
         private bool can_show_popup = false;
         private bool cursor_over_trayicon = false;
+        private bool show_notifications = false;
+        private TrackInfo current_track = null;
 
         private static readonly uint SkipDelta = 10;
         private static readonly int VolumeDelta = 10;
         
-        protected override void PluginInitialize() {
+        protected override void PluginInitialize() 
+        {
             Init();
 
             ui_manager_id = Globals.ActionManager.UI.AddUiFromResource("NotificationAreaIconMenu.xml");        
@@ -89,9 +89,16 @@ namespace Banshee.Plugins.NotificationAreaIcon {
             if (PlayerEngineCore.CurrentState == PlayerEngineState.Playing) {
                 FillPopup();
             }
+
+            Notifications.Widget = event_box;
+
+            // Forcefully load this value
+            show_notifications = ShowNotifications;
         }
 
-        protected override void PluginDispose() {
+        protected override void PluginDispose() 
+        {
+            Notifications.Widget = null;
             if (notif_area != null) {
                 notif_area.Destroy();
                 event_box = null;
@@ -101,7 +108,18 @@ namespace Banshee.Plugins.NotificationAreaIcon {
             Globals.ActionManager.UI.RemoveUi(ui_manager_id);
         }
 
-        private void Init() {
+        public override Gtk.Widget GetConfigurationWidget()
+        {            
+            return new NotificationAreaIconConfigPage(this);
+        }
+
+        protected override void InterfaceInitialize() 
+        {
+            InterfaceElements.MainWindow.KeyPressEvent += OnKeyPressEvent;
+        }
+
+        private void Init() 
+        {
             notif_area = new NotificationArea(Catalog.GetString("Banshee"));
             notif_area.DestroyEvent += OnDestroyEvent;
 
@@ -112,35 +130,96 @@ namespace Banshee.Plugins.NotificationAreaIcon {
             event_box.ScrollEvent += OnMouseScroll;
             
             event_box.Add(new Gtk.Image(IconThemeUtils.LoadIcon(22, "music-player-banshee", "tray-icon")));
-            
+
             notif_area.Add(event_box);
             notif_area.ShowAll();
         }
 
-        private void OnDestroyEvent(object o, DestroyEventArgs args) {
+        private void OnDestroyEvent(object o, DestroyEventArgs args) 
+        {
             Init();
         }
 
-        private void OnNotificationAreaIconClick(object o, ButtonPressEventArgs args) {
+        private void ShowHideMainWindow()
+        {
+            if (InterfaceElements.MainWindow.IsActive) {
+                InterfaceElements.MainWindow.Visible = false;
+            } else {
+                InterfaceElements.MainWindow.Present();
+            }
+        }
+
+        private void ShowNotification()
+        {
+            if (cursor_over_trayicon || !show_notifications) {
+                return;
+            }
+            
+            string message = String.Format("{0}\n<i>{1}</i>", 
+                    GLib.Markup.EscapeText(current_track.DisplayTitle),
+                    GLib.Markup.EscapeText(current_track.DisplayArtist));
+            Gdk.Pixbuf image = null;
+            
+            if (current_track.CoverArtFileName != null) {
+                image = new Gdk.Pixbuf(current_track.CoverArtFileName);
+            } else {
+                image = Branding.DefaultCoverArt;
+            }
+            
+            Notifications.Notify(Catalog.GetString("Now Playing"), message, image);    
+        }
+
+        [GLib.ConnectBefore]
+        private void OnKeyPressEvent(object o, KeyPressEventArgs args)
+        {
+            bool handled = false;
+            
+            if (args.Event.Key == Gdk.Key.w && (args.Event.State & Gdk.ModifierType.ControlMask) != 0) {
+                handled = true;
+                ShowHideMainWindow();
+                ResizeMoveWindow();
+            }
+            
+            args.RetVal = handled;
+        }
+
+        private void OnNotificationAreaIconClick(object o, ButtonPressEventArgs args) 
+        {
+            if(args.Event.Type != Gdk.EventType.ButtonPress) {
+                return;
+            }
+        
             switch(args.Event.Button) {
                 case 1:
-                    InterfaceElements.MainWindow.Visible = !InterfaceElements.MainWindow.Visible;
-                    ResizeMoveWindow();
+                    if((args.Event.State & Gdk.ModifierType.ControlMask) != 0) {
+                        Globals.ActionManager["PreviousAction"].Activate();
+                    } else {
+                        ShowHideMainWindow();
+                        ResizeMoveWindow();
+                    }
+                    break;
+                case 2:
+                    Globals.ActionManager["PlayPauseAction"].Activate();
                     break;
                 case 3:
-                    menu.Popup(null, null,
-                            new MenuPositionFunc(PositionMenu),
+                    if((args.Event.State & Gdk.ModifierType.ControlMask) != 0) {
+                        Globals.ActionManager["NextAction"].Activate();
+                    } else {
+                        menu.Popup(null, null, new MenuPositionFunc(PositionMenu), 
                             args.Event.Button, args.Event.Time);
+                    }
                     break;
             }
         }
 
-        private void PositionMenu(Menu menu, out int x, out int y, out bool push_in) {
+        private void PositionMenu(Menu menu, out int x, out int y, out bool push_in) 
+        {
             PositionWidget(menu, out x, out y, 0);
             push_in = true;
         }
 
-        private void PositionWidget(Widget widget, out int x, out int y, int yPadding) {
+        private void PositionWidget(Widget widget, out int x, out int y, int yPadding) 
+        {
             int button_y, panel_width, panel_height;
             Gtk.Requisition requisition = widget.SizeRequest();
             
@@ -152,7 +231,8 @@ namespace Banshee.Plugins.NotificationAreaIcon {
                 : button_y + panel_height + yPadding;
         }
 
-        private void OnMouseScroll(object o, ScrollEventArgs args) {
+        private void OnMouseScroll(object o, ScrollEventArgs args) 
+        {
             switch(args.Event.Direction) {
                 case Gdk.ScrollDirection.Up:
                     if((args.Event.State & Gdk.ModifierType.ControlMask) != 0) {
@@ -176,32 +256,39 @@ namespace Banshee.Plugins.NotificationAreaIcon {
                     }
                     break;
             }
-
         }
 
-        private void HidePopup() {
+        private void HidePopup() 
+        {
             popup.Hide();
         }
         
-        private void ShowPopup() {
+        private void ShowPopup() 
+        {
             PositionPopup();
             popup.Show();
         }
         
-        private void PositionPopup() {
+        private void PositionPopup() 
+        {
             int x, y;
             Gtk.Requisition event_box_req = event_box.SizeRequest();
             Gtk.Requisition popup_req = popup.SizeRequest();
+            
             PositionWidget(popup, out x, out y, 5);
+            
             x = x - (popup_req.Width / 2) + (event_box_req.Width / 2);     
             if(x + popup_req.Width >= event_box.Screen.Width) { 
                 x = event_box.Screen.Width - popup_req.Width - 5;
+            } else if(x < 5) {
+                x = 5;
             }
             
             popup.Move(x, y);
         }
         
-        private void OnEnterNotifyEvent(object o, EnterNotifyEventArgs args) {
+        private void OnEnterNotifyEvent(object o, EnterNotifyEventArgs args) 
+        {
             cursor_over_trayicon = true;
             if(can_show_popup) {
                 // only show the popup when the cursor is still over the
@@ -215,17 +302,24 @@ namespace Banshee.Plugins.NotificationAreaIcon {
             }
         }
         
-        private void OnLeaveNotifyEvent(object o, LeaveNotifyEventArgs args) {
+        private void OnLeaveNotifyEvent(object o, LeaveNotifyEventArgs args) 
+        {
             cursor_over_trayicon = false;
             HidePopup();
         }
 
-        private void OnPlayerEngineEventChanged(object o, PlayerEngineEventArgs args) {
+        private void OnPlayerEngineEventChanged(object o, PlayerEngineEventArgs args) 
+        {
             switch (args.Event) {
                 case PlayerEngineEvent.Iterate:
                     if(PlayerEngineCore.CurrentTrack != null) {
                         popup.Duration = (uint)PlayerEngineCore.CurrentTrack.Duration.TotalSeconds;
                         popup.Position = PlayerEngineCore.Position;
+
+                        if (current_track != PlayerEngineCore.CurrentTrack) {
+                            current_track = PlayerEngineCore.CurrentTrack;
+                            ShowNotification();
+                        }
                     } else {
                         popup.Duration = 0;
                         popup.Position = 0;
@@ -249,7 +343,8 @@ namespace Banshee.Plugins.NotificationAreaIcon {
             }
         }
 
-        private void FillPopup() {
+        private void FillPopup() 
+        {
             can_show_popup = true;
             popup.Artist = PlayerEngineCore.CurrentTrack.DisplayArtist;
             popup.Album = PlayerEngineCore.CurrentTrack.DisplayAlbum;
@@ -261,8 +356,6 @@ namespace Banshee.Plugins.NotificationAreaIcon {
             }
         }
 
-
-
         //FIXME: GO AWAY UGLY COPY!!!
         private void ResizeMoveWindow() {
             Window WindowPlayer = InterfaceElements.MainWindow;
@@ -300,5 +393,20 @@ namespace Banshee.Plugins.NotificationAreaIcon {
             }
         }
         
+        public bool ShowNotifications {
+            get { 
+                try {
+                    show_notifications = (bool)Globals.Configuration.Get(GConfKeys.BasePath
+                            +ConfigurationName+"/ShowNotifications");
+                } catch (GConf.NoSuchKeyException) { 
+                    show_notifications = false;
+                }
+                return show_notifications; 
+            }
+            set { 
+                Globals.Configuration.Set(GConfKeys.BasePath+ConfigurationName+"/ShowNotifications", value);
+                show_notifications = value;
+            }
+        }
     }
 }
