@@ -1,9 +1,8 @@
-
 /***************************************************************************
  *  Main.cs
  *
- *  Copyright (C) 2005 Novell
- *  Written by Aaron Bockover (aaron@aaronbock.net)
+ *  Copyright (C) 2005-2006 Novell, Inc.
+ *  Written by Aaron Bockover <aaron@abock.org>
  ****************************************************************************/
 
 /*  THIS FILE IS LICENSED UNDER THE MIT LICENSE AS OUTLINED IMMEDIATELY BELOW: 
@@ -40,20 +39,7 @@ namespace Banshee
     {    
         public static void Main(string [] args)
         {
-#if EXCEPTION_DIALOG        
-            try {
-                Startup(args);
-            } catch(Exception e) {
-                Console.Error.WriteLine(e);
-                Gtk.Application.Init();
-                ExceptionDialog dlg = new ExceptionDialog(e);
-                dlg.Run();
-                dlg.Destroy();
-                System.Environment.Exit(1);
-            }
-#else
-            Startup(args);
-#endif
+            Banshee.Gui.CleanRoomStartup.Startup(Startup, args);
         }
         
         [System.Runtime.InteropServices.DllImport("libdbus-glib-1")]
@@ -77,12 +63,15 @@ namespace Banshee
                 new ArgumentLayout("toggle-playing", "Toggle playing of current song"),
                 new ArgumentLayout("play",           "Play current song"),
                 new ArgumentLayout("pause",          "Pause current song"),
+                new ArgumentLayout("shutdown",       "Shutdown Banshee"),
                 new ArgumentLayout("query-artist",   "Get artist name for current playing song"),
                 new ArgumentLayout("query-album",    "Get album name for current playing song"),
                 new ArgumentLayout("query-title",    "Get track title for current playing song"),
                 new ArgumentLayout("query-genre",    "Get track genre for current playing song"),
                 new ArgumentLayout("query-duration", "Get duration of current playing song in seconds"),
                 new ArgumentLayout("query-position", "Get position of current playing song in seconds"),
+                new ArgumentLayout("query-rating",   "Get track rating for current playing song"),
+                new ArgumentLayout("set-rating",     "Set track rating for current playing song"),
                 new ArgumentLayout("query-uri",      "Get URI of current playing song"),
                 new ArgumentLayout("query-cover-uri","Get URI of the album cover of current playing song"),
                 new ArgumentLayout("query-status",   "Get player status (-1: Not loaded, 0: Paused, 1: Playing)"),
@@ -94,21 +83,21 @@ namespace Banshee
             }, args, "enqueue");
             
             HandleShallowCommands();
-            RemotePlayer dbus_core = DetectInstanceAndDbus();
+            DBusPlayer dbus_core = DetectInstanceAndDbus();
             HandleDbusCommands(dbus_core);
             
             Globals.Initialize();
             StockIcons.Initialize();
             new Program(StringUtil.UcFirst(ConfigureDefines.PACKAGE), ConfigureDefines.VERSION, Modules.UI, args);
-            PlayerCore.UserInterface = new Banshee.PlayerUI();
+            new Banshee.PlayerUI();
             
             Gtk.Application.Run();
         }
     
-        private static RemotePlayer DetectInstanceAndDbus()
+        private static DBusPlayer DetectInstanceAndDbus()
         {
             try {
-                return RemotePlayer.FindInstance();
+                return DBusPlayer.FindInstance();
             } catch {
                 /*Process current_process = Process.GetCurrentProcess();
                 foreach(Process process in Process.GetProcesses()) {
@@ -125,7 +114,7 @@ namespace Banshee
             }
         }
         
-        private static void HandleDbusCommands(RemotePlayer remote_player)
+        private static void HandleDbusCommands(DBusPlayer remote_player)
         {
             if(remote_player == null) {
                 return;
@@ -141,6 +130,10 @@ namespace Banshee
             foreach(string arg in Globals.ArgumentQueue.Arguments) {
                 bool dequeue = true;
                 switch(arg) {
+                case "shutdown":
+                    remote_player.Shutdown();
+                    present = false;
+                    break;
                 case "toggle-playing":
                     remote_player.TogglePlaying();
                     present = false;
@@ -193,6 +186,10 @@ namespace Banshee
                     PrintQueryResult("Position", remote_player.GetPlayingPosition());
                     present = false;
                     break;
+                case "query-rating":
+                    PrintQueryResult("Rating", remote_player.GetPlayingRating());
+                    present = false;
+                    break;
                 case "query-uri":
                     PrintQueryResult("Uri", remote_player.GetPlayingUri());
                     present = false;
@@ -211,8 +208,17 @@ namespace Banshee
                     try {
                         position = Convert.ToInt32(string_position);
                         remote_player.SetPlayingPosition(position);
-                    } catch(Exception) {
-                        Console.WriteLine("Invalid position `{0}'. Integer value expected.", string_position);
+                    } catch {
+                        Console.Error.WriteLine("Invalid position `{0}'. Integer value expected.", string_position);
+                    }
+                    present = false;
+                    break;
+                case "set-rating":
+                    string rating = Globals.ArgumentQueue[arg];
+                    try {
+                        remote_player.SetPlayingRating(Convert.ToInt32(rating));
+                    } catch {
+                        Console.Error.WriteLine("Invalid rating `{0}'. Integer value expected.", rating);
                     }
                     present = false;
                     break;
@@ -259,7 +265,7 @@ namespace Banshee
             }
         }
         
-        private static void Present(RemotePlayer remote_player)
+        private static void Present(DBusPlayer remote_player)
         {
             remote_player.PresentWindow();
         }
