@@ -58,6 +58,7 @@ namespace Banshee
             Title,
             Album,
             Genre,
+            Year,
             Time,
             Rating,
             PlayCount,
@@ -71,6 +72,7 @@ namespace Banshee
         Pixbuf nowPlayingPixbuf;
         Pixbuf songDrmedPixbuf;
         Pixbuf ripColumnPixbuf;
+        Banshee.Gui.CellRendererRating rating_renderer;
 
         public TreeViewColumn RipColumn;
         public PlaylistColumn RatingColumn;
@@ -85,6 +87,10 @@ namespace Banshee
             }
         }
         
+        protected PlaylistView(IntPtr raw) : base(raw)
+        {
+        }
+        
         public PlaylistView(PlaylistModel model)
         {        
             // set up columns
@@ -105,28 +111,33 @@ namespace Banshee
             columns.Add(new PlaylistColumn(this, Catalog.GetString("Genre"), "Genre", 
                 new TreeCellDataFunc(TrackCellGenre), new CellRendererText(),
                 4, (int)ColumnId.Genre));
-            columns.Add(new PlaylistColumn(this, Catalog.GetString("Time"), "Time", 
+            columns.Add(new PlaylistColumn(this, Catalog.GetString("Year"), "Year", 
+                new TreeCellDataFunc(TrackCellYear), new CellRendererText(),
+                5, (int)ColumnId.Year));
+           	columns.Add(new PlaylistColumn(this, Catalog.GetString("Time"), "Time", 
                 new TreeCellDataFunc(TrackCellTime), new CellRendererText(),
-                5, (int)ColumnId.Time));
+                6, (int)ColumnId.Time));
             
+            rating_renderer = new Banshee.Gui.CellRendererRating();
+            rating_renderer.RatingChanged += OnRatingChanged;
             RatingColumn = new PlaylistColumn(this, 
                 Catalog.GetString("Rating"), "Rating",
-                new TreeCellDataFunc(TrackCellRating), new RatingRenderer(),
-                6, (int)ColumnId.Rating);
+                new TreeCellDataFunc(TrackCellRating), rating_renderer,
+                7, (int)ColumnId.Rating);
             columns.Add(RatingColumn);
             
             PlaysColumn = new PlaylistColumn(this, 
                 Catalog.GetString("Plays"), "Plays",
                 new TreeCellDataFunc(TrackCellPlayCount), 
                 new CellRendererText(),
-                7, (int)ColumnId.PlayCount);
+                8, (int)ColumnId.PlayCount);
             columns.Add(PlaysColumn);
             
             LastPlayedColumn = new PlaylistColumn(this, 
                 Catalog.GetString("Last Played"), "Last-Played",
                 new TreeCellDataFunc(TrackCellLastPlayed), 
                 new CellRendererText(),
-                8, (int)ColumnId.LastPlayed);
+                9, (int)ColumnId.LastPlayed);
             columns.Add(LastPlayedColumn);
             
             columns.Sort(new ColumnSorter());
@@ -137,8 +148,8 @@ namespace Banshee
 
             // FIXME: would be nice to have these as PlaylistColumns too...
             TreeViewColumn playIndColumn = new TreeViewColumn();
-            Gtk.Image playIndImg = new Gtk.Image(
-                Gdk.Pixbuf.LoadFromResource("blue-speaker.png"));
+            Gtk.Image playIndImg = new Gtk.Image(IconThemeUtils.LoadIcon(16, "audio-volume-high", 
+                "blue-speaker"));
             playIndImg.Show();
             playIndColumn.Expand = false;
             playIndColumn.Resizable = false;
@@ -146,14 +157,14 @@ namespace Banshee
             playIndColumn.Reorderable = false;
             playIndColumn.Widget = playIndImg;
             
-            nowPlayingPixbuf = Gdk.Pixbuf.LoadFromResource("now-playing-arrow.png");
+            nowPlayingPixbuf = IconThemeUtils.LoadIcon(16, "media-playback-start", 
+                Stock.MediaPlay, "now-playing-arrow");
             songDrmedPixbuf = Gdk.Pixbuf.LoadFromResource("song-drm.png");
             ripColumnPixbuf = Gdk.Pixbuf.LoadFromResource("cd-action-rip-16.png");
             
             CellRendererPixbuf indRenderer = new CellRendererPixbuf();
             playIndColumn.PackStart(indRenderer, true);
-            playIndColumn.SetCellDataFunc(indRenderer, 
-                new TreeCellDataFunc(TrackCellInd));
+            playIndColumn.SetCellDataFunc(indRenderer, new TreeCellDataFunc(TrackCellInd));
             InsertColumn(playIndColumn, 0);
             
             RipColumn = new TreeViewColumn();
@@ -194,6 +205,8 @@ namespace Banshee
                 new TreeIterCompareFunc(AlbumTreeIterCompareFunc));
             model.SetSortFunc((int)ColumnId.Genre, 
                 new TreeIterCompareFunc(GenreTreeIterCompareFunc));
+            model.SetSortFunc((int)ColumnId.Year, 
+                new TreeIterCompareFunc(YearTreeIterCompareFunc));
             model.SetSortFunc((int)ColumnId.Time, 
                 new TreeIterCompareFunc(TimeTreeIterCompareFunc));
             model.SetSortFunc((int)ColumnId.Rating, 
@@ -215,6 +228,18 @@ namespace Banshee
             
             }   
         }
+        
+        private void OnRatingChanged(object o, Banshee.Gui.CellRatingChangedArgs args)
+        {
+            TreeIter iter;
+            if(model.GetIter(out iter, args.Path)) {
+                try {
+                    TrackInfo track = (TrackInfo)model.GetValue(iter, 0);
+                    track.Rating = args.Rating;
+                } catch {
+                }
+            }
+        }
 
         private bool CheckColumnDrop(TreeView tree, TreeViewColumn col,
                                      TreeViewColumn prev, TreeViewColumn next)
@@ -249,8 +274,13 @@ namespace Banshee
         public int ArtistTreeIterCompareFunc(TreeModel _model, TreeIter a, 
             TreeIter b)
         {
-            return StringFieldCompare(model.IterTrackInfo(a).Artist, 
+            int v = StringFieldCompare(model.IterTrackInfo(a).Artist, 
                 model.IterTrackInfo(b).Artist);
+            
+            if(v != 0)
+                return v;
+            
+            return AlbumTreeIterCompareFunc(_model, a, b);
         }
         
         public int TitleTreeIterCompareFunc(TreeModel _model, TreeIter a, 
@@ -266,10 +296,10 @@ namespace Banshee
             int v = StringFieldCompare(model.IterTrackInfo(a).Album, 
                 model.IterTrackInfo(b).Album);
 
-            if (v != 0)
+            if(v != 0)
                 return v;
 
-            return TrackTreeIterCompareFunc (_model, a, b);
+            return TrackTreeIterCompareFunc(_model, a, b);
         }
         
         public int GenreTreeIterCompareFunc(TreeModel _model, TreeIter a,
@@ -279,6 +309,13 @@ namespace Banshee
                 model.IterTrackInfo(b).Genre);
         }
         
+        public int YearTreeIterCompareFunc(TreeModel _model, TreeIter a,
+            TreeIter b)
+        {
+            return LongFieldCompare(model.IterTrackInfo(a).Year, 
+                model.IterTrackInfo(b).Year);
+        }
+        
         public int TimeTreeIterCompareFunc(TreeModel _model, TreeIter a,
             TreeIter b)
         {
@@ -418,9 +455,9 @@ namespace Banshee
             TrackInfo ti = model.IterTrackInfo(iter);
             if(ti == null) {
                 return;
-            }
-            
-            SetRendererAttributes((CellRendererText)cell, Convert.ToString(ti.TrackNumber), iter);
+            }            
+            SetRendererAttributes((CellRendererText)cell,
+			    ti.TrackNumber > 0 ? Convert.ToString(ti.TrackNumber) : String.Empty, iter);
         }    
         
         protected void TrackCellArtist(TreeViewColumn tree_column,
@@ -467,6 +504,18 @@ namespace Banshee
             SetRendererAttributes((CellRendererText)cell, ti.Genre, iter);
         }
         
+        protected void TrackCellYear(TreeViewColumn tree_column,
+            CellRenderer cell, TreeModel tree_model, TreeIter iter)
+        {
+            TrackInfo ti = model.IterTrackInfo(iter);
+            if(ti == null) {
+                return;
+            }
+            
+            int year = ti.Year;
+            SetRendererAttributes((CellRendererText)cell, year > 0 ? Convert.ToString(year) : String.Empty, iter);
+        }
+        
         protected void TrackCellTime(TreeViewColumn tree_column,
             CellRenderer cell, TreeModel tree_model, TreeIter iter)
         {
@@ -489,7 +538,7 @@ namespace Banshee
             }
             
             uint plays = ti.PlayCount;
-            SetRendererAttributes((CellRendererText)cell, plays > 0 ? Convert.ToString(plays) : "", iter);
+            SetRendererAttributes((CellRendererText)cell, plays > 0 ? Convert.ToString(plays) : String.Empty, iter);
         }
         
         protected void TrackCellRating(TreeViewColumn tree_column,
@@ -500,7 +549,7 @@ namespace Banshee
                 return;
             }
              
-            ((RatingRenderer)cell).Track = ti;
+            ((Banshee.Gui.CellRendererRating)cell).Rating = ti.Rating;
         }
         
         protected void TrackCellLastPlayed(TreeViewColumn tree_column,
@@ -556,6 +605,15 @@ namespace Banshee
                 ScrollToCell(model.PlayingPath, null, true, 0.5f, 0.0f);
             }
         }
+
+        public void SelectPlaying()
+        {
+            Selection.UnselectAll();
+            
+            if(model.PlayingPath != null) {
+                Selection.SelectPath(model.PlayingPath);
+            }
+        }
         
         public bool PlaySelected()
         {
